package owl.img.analysis;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.geom.Point2D;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.jfree.chart.ChartPanel;
import org.jfree.chart.panel.CrosshairOverlay;
import org.jfree.chart.plot.Crosshair;

import arc.api.device.ArcDeviceJAPI;
import arc.api.image.ArcImageJAPI;

import owl.img.buffer.ImageBufferListener;
import owl.img.buffer.ImageUpdateEvent;
import owl.main.MainApp;
import owl.main.owltypes.OwlNumberSpinner;
import owl.main.setup.SetupEvent;
import owl.main.setup.SetupFrame;
import owl.main.setup.SetupListener;
import owl.plot.PlotFactory;
import owl.plot.XYPlot;



public class AreaPlotFrame extends AnalysisFrame implements ChangeListener, ItemListener, SetupListener, ImageBufferListener
{
	private static final long serialVersionUID = -3250377878473214959L;

	protected final String AREA_ROW_PLOT	 = "Area Row Plot";
	protected final String AREA_COL_PLOT	 = "Area Col Plot";

	protected OwlNumberSpinner	m_rowStartSpinner;
	protected OwlNumberSpinner	m_rowEndSpinner;
	protected OwlNumberSpinner	m_colStartSpinner;
	protected OwlNumberSpinner	m_colEndSpinner;
	protected CrosshairOverlay	m_crosshairOverlay;
	protected XYPlot			m_xyAreaRowPlot;
	protected XYPlot			m_xyAreaColPlot;
	protected String			m_bActivePlot;

	private   boolean			m_bChangesEnabled;
	private   Object			m_syncLock;


	public AreaPlotFrame()
	{
		super( "Area Plot" );

		m_syncLock			= new Object();
		m_bChangesEnabled	= true;

		super.addComponent( createCtrlPanel(),
							super.SOUTH_CONTAINER_INDEX );

		Crosshair crosshair = new Crosshair();

		m_crosshairOverlay = new CrosshairOverlay();
		m_crosshairOverlay.addDomainCrosshair( crosshair );
		m_crosshairOverlay.addRangeCrosshair( crosshair );

		m_xyAreaRowPlot = new XYPlot();
		m_xyAreaRowPlot.setXLabel( "Row" );
		m_xyAreaRowPlot.setYLabel( "ADU" );

		ChartPanel areaRowChartPanel =
							PlotFactory.createXYChart( m_xyAreaRowPlot,
													   "Area Row Plot",
													   Color.white,
													   true );

		areaRowChartPanel.addChartMouseListener( this );
		areaRowChartPanel.addOverlay( m_crosshairOverlay );

		m_xyAreaColPlot = new XYPlot();
		m_xyAreaColPlot.setXLabel( "Column" );
		m_xyAreaColPlot.setYLabel( "ADU" );

		ChartPanel areaColChartPanel =
							PlotFactory.createXYChart( m_xyAreaColPlot,
													   "Area Column Plot",
													   Color.white,
													   true );

		areaColChartPanel.addChartMouseListener( this );
		areaColChartPanel.addOverlay( m_crosshairOverlay );

		JTabbedPane tabPane = new JTabbedPane();
		tabPane.addChangeListener( this );
		tabPane.addTab( AREA_ROW_PLOT, areaRowChartPanel );
		tabPane.addTab( AREA_COL_PLOT, areaColChartPanel );

		super.setSize( new Dimension( 875, 600 ) );
		super.setComponent( tabPane );
		super.pack();

		loadPreferences();

		SetupFrame.addSetupListener( this );
		ImageUpdateEvent.addImageBufferListener( this );

		( new Thread( new ReadCtlrRunnable() ) ).start();
	}

	// +-----------------------------------------------------------------------------------+
	// | SetupListener method overrides                                                    |
	// +-----------------------------------------------------------------------------------+
	@Override
	public void setupChanged( SetupEvent event )
	{
		if ( event.bSetupOk )
		{
			setImageRowsValue( event.dRows );
			setImageColsValue( event.dCols );

			m_rowStartSpinner.setMaximum( event.dRows );
			m_rowEndSpinner.setMaximum( event.dRows );

			m_colStartSpinner.setMaximum( event.dCols );
			m_colEndSpinner.setMaximum( event.dCols );

			if ( ( ( Integer )m_rowStartSpinner.getValue() ).intValue() > event.dRows )
			{
				m_rowStartSpinner.setValue( Integer.valueOf( event.dRows ) );
			}

			if ( ( ( Integer )m_rowEndSpinner.getValue() ).intValue() > event.dRows ||
			   ( ( ( Integer )m_rowEndSpinner.getValue() ).intValue() == 0 ) )
			{
				m_rowEndSpinner.setValue( Integer.valueOf( event.dRows ) );
			}

			if ( ( ( Integer )m_colStartSpinner.getValue() ).intValue() > event.dCols )
			{
				m_colStartSpinner.setValue( Integer.valueOf( event.dCols ) );
			}

			if ( ( ( Integer )m_colEndSpinner.getValue() ).intValue() > event.dCols ||
			   ( ( ( Integer )m_colEndSpinner.getValue() ).intValue() == 0 ) )
			{
				m_colEndSpinner.setValue( Integer.valueOf( event.dCols ) );
			}
		}
	}

	// +-----------------------------------------------------------------------------------+
	// | ChangeListener method overrides                                                   |
	// +-----------------------------------------------------------------------------------+
	@Override
	public void stateChanged( ChangeEvent ce )
	{
		if ( ce.getSource() instanceof javax.swing.JTabbedPane )
		{
			JTabbedPane tabPane = ( JTabbedPane )ce.getSource();

			m_bActivePlot =
				tabPane.getTitleAt( tabPane.getSelectedIndex() );
		}

		else if ( m_bChangesEnabled )
		{
			if ( ( ce.getSource() instanceof javax.swing.JSpinner ) && this.isVisible() )
			{
				( new Thread( this ) ).start();
			}
		}
	}

	// +-----------------------------------------------------------------------------------+
	// | ItemListener method overrides                                                     |
	// +-----------------------------------------------------------------------------------+
	@Override
	public void itemStateChanged( ItemEvent arg0 )
	{
		if ( super.isShowCrosshair() )
		{
			m_xyAreaRowPlot.crosshairOverlayEnabled( true );
			m_xyAreaColPlot.crosshairOverlayEnabled( true );
		}
		else
		{
			m_xyAreaRowPlot.crosshairOverlayEnabled( false );
			m_xyAreaColPlot.crosshairOverlayEnabled( false );
		}

		m_crosshairOverlay.fireOverlayChanged();
	}

	// +-----------------------------------------------------------------------------------+
	// | ImageBufferListener method overrides                                              |
	// +-----------------------------------------------------------------------------------+
	@Override
	public void bufferChanged( ImageUpdateEvent event )
	{
		boolean bAuto = false;

		enableChangeListeners( false );

		m_rowStartSpinner.setMaximum( ( int )event.getRows() );

		m_rowEndSpinner.setMaximum( ( int )event.getRows() );

		if ( ( ( Integer )m_rowEndSpinner.getValue() ).intValue() > event.getCols() )
		{
			m_rowEndSpinner.setValue( Integer.valueOf( ( int )event.getRows() ) );

			bAuto = true;
		}

		m_colStartSpinner.setMaximum( ( int )event.getCols() );

		m_colEndSpinner.setMaximum( ( int )event.getCols() );

		if ( ( ( Integer )m_colEndSpinner.getValue() ).intValue() > event.getCols() )
		{
			m_colEndSpinner.setValue( Integer.valueOf( ( int )event.getCols() ) );

			bAuto = true;
		}

		m_rowsTxtfld.setValue( event.getRows() );
		m_colsTxtfld.setValue( event.getCols() );

		enableChangeListeners( true );

		if ( this.isVisible() )
		{
			( new Thread( this ) ).start();
		}

		if ( bAuto )
		{
			setAutoRange();
		}
	}

	// +-----------------------------------------------------------------------------------+
	// | AnalysisFrame method overrides                                                    |
	// +-----------------------------------------------------------------------------------+
	@Override
	public void readCtlr()
	{
		enableChangeListeners( false );

		try
		{
			if ( ArcDeviceJAPI.IsOpen() && ArcDeviceJAPI.IsControllerConnected() )
			{
				MainApp.infoStart( "Reading image dimensions from controller" );

				int dRowEnd = ArcDeviceJAPI.GetImageRows();
				int dColEnd = ArcDeviceJAPI.GetImageCols();

				setImageRowsValue( dRowEnd );
				setImageColsValue( dColEnd );

				m_rowStartSpinner.setMaximum( dRowEnd );
				m_rowStartSpinner.setValue( Integer.valueOf( 0 ) );

				m_rowEndSpinner.setMaximum( dRowEnd );
				m_rowEndSpinner.setValue( dRowEnd );

				m_colStartSpinner.setMaximum( dColEnd );
				m_colStartSpinner.setValue( Integer.valueOf( 0 ) );

				m_colEndSpinner.setMaximum( dColEnd );
				m_colEndSpinner.setValue( dColEnd );
						
				MainApp.infoEnd();
			}
			else
			{
				MainApp.error( "( ReadCtlrRunnable ): " +
							   "No device or controller connected!" );
			}
		}
		catch ( Throwable e )
		{
			MainApp.error( e );
		}

		super.pack();

		enableChangeListeners( true );
	}

	@Override
	public void readDS9()
	{
		try
		{
			MainApp.info( "Reading image dimensions from DS9 ...." );
			MainApp.warn( "Requires POINT shape to be manually selected within DS9!" );

			long[] lCoords = getDS9Box();

			if ( m_bStop ) { m_runButton.setBlack( RUN_ACTION ); return; }

			long lRowStart = ( lCoords[ BOX_CENTER_ROW ] - ( lCoords[ BOX_ROWS ] / 2 ) );
			long lRowEnd   = ( lCoords[ BOX_CENTER_ROW ] + ( lCoords[ BOX_ROWS ] / 2 ) );
			long lColStart = ( lCoords[ BOX_CENTER_COL ] - ( lCoords[ BOX_COLS ] / 2 ) );
			long lColEnd   = ( lCoords[ BOX_CENTER_COL ] + ( lCoords[ BOX_COLS ] / 2 ) );

			setImageRowsValue( MainApp.getMainFrame().getCameraPanel().getRowLabelValue() );
			setImageColsValue( MainApp.getMainFrame().getCameraPanel().getColLabelValue() );

			m_rowStartSpinner.setValue( ( new Long( lRowStart ) ).intValue() );
			m_rowEndSpinner.setValue( ( new Long( lRowEnd ) ).intValue() );

			m_colStartSpinner.setValue( ( new Long( lColStart ) ).intValue() );
			m_colEndSpinner.setValue( ( new Long( lColEnd ) ).intValue() );
		}
		catch ( Throwable e )
		{
			MainApp.error( e );
		}
	}

	@Override
	public void setAutoRange()
	{
		if ( m_bActivePlot.equals( AREA_ROW_PLOT ) )
		{
			m_xyAreaRowPlot.getDomainAxis().setAutoRange( true );
			m_xyAreaRowPlot.getRangeAxis().setAutoRange( true );
		}
		else
		{
			m_xyAreaColPlot.getDomainAxis().setAutoRange( true );
			m_xyAreaColPlot.getRangeAxis().setAutoRange( true );
		}
	}

	@Override
	public void run()
	{
		Thread.currentThread().setName( "Owl - AreaPlotFrame ( " + m_bActivePlot + " )" );

		m_runButton.setColored( ABORT_ACTION );

		try
		{
			synchronized( m_syncLock )
			{
				Point2D.Float[] points = null;

				if ( m_bStop )
				{
					throw new Exception( "Plot update aborted! You should not see this!" );
				}
	
				//
				// Do an area row plot
				//
				if ( m_bActivePlot.equals( AREA_ROW_PLOT ) )
				{
					points = ArcImageJAPI.GetRowArea( ArcDeviceJAPI.GetCommonBufferVA(),
													  m_rowStartSpinner.getInt(),
													  m_rowEndSpinner.getInt(),
													  m_colStartSpinner.getInt(),
													  m_colEndSpinner.getInt(),
													  super.getImageRowsValue(),
													  super.getImageColsValue() );
	
					m_xyAreaRowPlot.setPlot( points,
											 "Area Row [ " + m_rowStartSpinner.getInt() +
											 ":" + m_rowEndSpinner.getInt() + " ]" );
				}
	
				//
				// Do an area column plot
				//
				else if ( m_bActivePlot.equals( AREA_COL_PLOT ) )
				{
					points = ArcImageJAPI.GetColArea( ArcDeviceJAPI.GetCommonBufferVA(),
													  m_rowStartSpinner.getInt(),
													  m_rowEndSpinner.getInt(),
													  m_colStartSpinner.getInt(),
													  m_colEndSpinner.getInt(),
													  super.getImageRowsValue(),
													  super.getImageColsValue() );
	
					m_xyAreaColPlot.setPlot( points,
											 "Area Col [ " + m_colStartSpinner.getInt() +
											 ":" + m_colEndSpinner.getInt() + " ]" );
				}
	
				//
				// Error, unknown plot type
				//
				else
				{
					throw new Exception( "Invalid area plot type: " + m_bActivePlot );
				}
	
				if ( m_bStop )
				{
					throw new Exception( "Plot update aborted! You should not see this!" );
				}
	
				//
				// Get area statistics
				//
				super.getStats( m_rowStartSpinner.getInt(),
								m_rowEndSpinner.getInt(),
								m_colStartSpinner.getInt(),
								m_colEndSpinner.getInt(),
								super.getImageRowsValue(),
								super.getImageColsValue() );
			}	// end of synchonization
		}
		catch ( Throwable e )
		{
			if ( !m_bStop )
			{
				MainApp.error( e.getMessage() );
			}
		}
		finally
		{
			m_runButton.setBlack( RUN_ACTION );
		}

		super.pack();
	}

	// +-----------------------------------------------------------------------------------+
	// | createCtrlPanel                                                                   |
	// +-----------------------------------------------------------------------------------+
	// | Creates the plot control ( input ) panel components.                              |
	// +-----------------------------------------------------------------------------------+
	private JPanel createCtrlPanel()
	{
		JPanel rPanel = new JPanel();
		rPanel.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.RAISED ) );

		m_rowStartSpinner = new OwlNumberSpinner( 6 );
		m_rowStartSpinner.addChangeListener( this );

		m_rowEndSpinner = new OwlNumberSpinner( 6 );
		m_rowEndSpinner.addChangeListener( this );

		rPanel.add( new JLabel( "<html><b>ROW</b></html>" ) );
		rPanel.add( javax.swing.Box.createHorizontalStrut( 3 ) );
		rPanel.add( new JLabel( "start: " ) );
		rPanel.add( m_rowStartSpinner );
		rPanel.add( javax.swing.Box.createHorizontalStrut( 3 ) );
		rPanel.add( new JLabel( "end: " ) );
		rPanel.add( m_rowEndSpinner );

		JPanel cPanel = new JPanel();
		cPanel.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.RAISED ) );

		m_colStartSpinner = new OwlNumberSpinner( 6 );
		m_colStartSpinner.addChangeListener( this );

		m_colEndSpinner = new OwlNumberSpinner( 6 );
		m_colEndSpinner.addChangeListener( this );

		cPanel.add( new JLabel( "<html><b>COL</b></html>" ) );
		cPanel.add( javax.swing.Box.createHorizontalStrut( 3 ) );
		cPanel.add( new JLabel( "start: " ) );
		cPanel.add( m_colStartSpinner );
		cPanel.add( javax.swing.Box.createHorizontalStrut( 3 ) );
		cPanel.add( new JLabel( "end: " ) );
		cPanel.add( m_colEndSpinner );

		JPanel sPanel = createStatsPanel( true, this );

		JPanel panel = new JPanel();
		panel.setBorder( BorderFactory.createLoweredBevelBorder() );
		panel.setLayout( super.gbl );

		addComponent( panel, rPanel,              0, 0, 0, 0, 0, 0, 1, 1 );
		addComponent( panel, cPanel,              0, 0, 0, 0, 0, 1, 1, 1 );
		addComponent( panel, getCoordsSrcPanel(), 0, 0, 0, 0, 0, 2, 1, 1 );
		addComponent( panel, sPanel,              0, 0, 0, 0, 1, 0, 1, 3 );

		return panel;
	}

	// +-----------------------------------------------------------------------------------+
	// | enableChangeListeners                                                             |
	// +-----------------------------------------------------------------------------------+
	// | Enables/Disables the change listeners for the JSpinner components in this window. |
	// | Used to disable spinner change listeners at startup.                              |
	// +-----------------------------------------------------------------------------------+
	private void enableChangeListeners( boolean bEnable )
	{
		m_bChangesEnabled = bEnable;
	}
}
