package owl.img.analysis;

import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.geom.Point2D;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.jfree.chart.ChartPanel;
import org.jfree.chart.panel.CrosshairOverlay;
import org.jfree.chart.plot.Crosshair;

import arc.api.device.ArcDeviceJAPI;
import arc.api.image.ArcImageJAPI;

import owl.img.buffer.ImageBufferListener;
import owl.img.buffer.ImageUpdateEvent;
import owl.main.MainApp;
import owl.main.owltypes.OwlNumberSpinner;
import owl.main.setup.SetupEvent;
import owl.main.setup.SetupListener;
import owl.plot.PlotFactory;
import owl.plot.XYPlot;



public class ColPlotFrame extends AnalysisFrame implements ItemListener, SetupListener, ImageBufferListener, ChangeListener
{
	private static final long serialVersionUID = -6715850711759429614L;

	protected OwlNumberSpinner	m_colSpinner;
	protected OwlNumberSpinner	m_rowStartSpinner;
	protected OwlNumberSpinner	m_rowEndSpinner;
	protected CrosshairOverlay	m_crosshairOverlay;
	protected XYPlot			m_xyPlot;

	private   boolean			m_bChangesEnabled;
	private   Object			m_syncLock;


	public ColPlotFrame()
	{
		super( "Column Plot" );

		m_syncLock			= new Object();
		m_bChangesEnabled	= true;

		m_xyPlot = new XYPlot();
		m_xyPlot.setXLabel( "Row" );
		m_xyPlot.setYLabel( "ADU" );

		Crosshair crosshair = new Crosshair();

		m_crosshairOverlay = new CrosshairOverlay();
		m_crosshairOverlay.addDomainCrosshair( crosshair );
		m_crosshairOverlay.addRangeCrosshair( crosshair );

		ChartPanel chartPanel = PlotFactory.createXYChart( m_xyPlot,
														   super.getTitle(),
														   Color.white,
														   true );
		chartPanel.addChartMouseListener( this );
		chartPanel.addOverlay( m_crosshairOverlay );

		super.setPlot( chartPanel );

		super.addComponent( createCtrlPanel(),
							super.SOUTH_CONTAINER_INDEX );

		super.pack();

		loadPreferences();

		owl.main.setup.SetupFrame.addSetupListener( this );

		ImageUpdateEvent.addImageBufferListener( this );

		( new Thread( new ReadCtlrRunnable() ) ).start();
	}

	// +-----------------------------------------------------------------------------------+
	// | SetupListener method overrides                                                    |
	// +-----------------------------------------------------------------------------------+
	@Override
	public void setupChanged( SetupEvent event )
	{
		if ( event.bSetupOk )
		{
			setImageRowsValue( event.dRows );
			setImageColsValue( event.dCols );

			m_colSpinner.setMaximum( event.dCols );

			m_rowStartSpinner.setMaximum( event.dRows );
			m_rowEndSpinner.setMaximum( event.dRows );

			if ( ( ( Integer )m_colSpinner.getValue() ).intValue() > event.dCols )
			{
				m_colSpinner.setValue( Integer.valueOf( event.dCols ) );
			}

			if ( ( ( Integer )m_rowStartSpinner.getValue() ).intValue() > event.dRows )
			{
				m_rowStartSpinner.setValue( Integer.valueOf( event.dRows ) );
			}

			if ( ( ( Integer )m_rowEndSpinner.getValue() ).intValue() > event.dRows ||
			   ( ( ( Integer )m_rowEndSpinner.getValue() ).intValue() == 0 ) )
			{
				m_rowEndSpinner.setValue( Integer.valueOf( event.dRows ) );
			}
		}
	}

	// +-----------------------------------------------------------------------------------+
	// | ImageBufferListener method overrides                                              |
	// +-----------------------------------------------------------------------------------+
	@Override
	public void bufferChanged( ImageUpdateEvent event )
	{
		boolean bAuto = false;

		enableChangeListeners( false );

		m_rowEndSpinner.setMaximum( ( int )event.getRows() );

		if ( ( ( Integer )m_rowEndSpinner.getValue() ).intValue() > event.getRows() )
		{
			m_rowEndSpinner.setValue( Integer.valueOf( ( int )event.getRows() ) );

			bAuto = true;
		}

		m_colSpinner.setMaximum( ( int )event.getCols() );

		m_rowsTxtfld.setValue( event.getRows() );
		m_colsTxtfld.setValue( event.getCols() );

		enableChangeListeners( true );

		if ( this.isVisible() )
		{
			( new Thread( this ) ).start();
		}

		if ( bAuto )
		{
			setAutoRange();
		}
	}

	// +-----------------------------------------------------------------------------------+
	// | ItemListner method overrides                                                      |
	// +-----------------------------------------------------------------------------------+
	@Override
	public void itemStateChanged( ItemEvent arg0 )
	{
		if ( m_linesChkbox.isSelected() )
		{
			m_xyPlot.crosshairOverlayEnabled( true );
		}

		else
		{
			m_xyPlot.crosshairOverlayEnabled( false );
		}

		m_crosshairOverlay.fireOverlayChanged();
	}

	// +-----------------------------------------------------------------------------------+
	// | ChangeListener method overrides                                                   |
	// +-----------------------------------------------------------------------------------+
	@Override
	public void stateChanged( ChangeEvent ce )
	{
		if ( m_bChangesEnabled )
		{
			if ( ( ce.getSource() instanceof javax.swing.JSpinner ) && this.isVisible() )
			{
				( new Thread( this ) ).start();
			}
		}
	}

	// +-----------------------------------------------------------------------------------+
	// | AnalysisFrame method overrides                                                    |
	// +-----------------------------------------------------------------------------------+
	@Override
	public void readCtlr()
	{
		enableChangeListeners( false );

		try
		{
			if ( ArcDeviceJAPI.IsOpen() && ArcDeviceJAPI.IsControllerConnected() )
			{
				MainApp.infoStart( "Reading image dimensions from controller" );

				int dRowEnd = ArcDeviceJAPI.GetImageRows();
				int dColEnd = ArcDeviceJAPI.GetImageCols();

				//  Set image dimensions first because the spinners already
				//  have active change listeners and will see the row and
				//  col values with zeros on startup!
				// +---------------------------------------------------------+
				setImageRowsValue( dRowEnd );
				setImageColsValue( dColEnd );

				m_rowStartSpinner.setMaximum( dRowEnd );
				m_rowStartSpinner.setValue( Integer.valueOf( 0 ) );

				m_rowEndSpinner.setMaximum( dRowEnd );
				m_rowEndSpinner.setValue( dRowEnd );

				m_colSpinner.setMaximum( dColEnd );
				m_colSpinner.setValue( Integer.valueOf( 0 ) );

				MainApp.infoEnd();
			}
			else
			{
				MainApp.error( "( ReadCtlrRunnable ): " +
							   "No device or controller connected!" );
			}
		}
		catch ( Throwable e )
		{
			MainApp.error( e );
		}

		super.pack();

		enableChangeListeners( true );
	}

	@Override
	public void readDS9()
	{
		try
		{
			MainApp.info( "Reading image dimensions from DS9 ...." );
			MainApp.warn( "Requires POINT shape to be manually selected within DS9!" );

			long[] lCoords = getDS9Point();

			if ( m_bStop ) { m_runButton.setBlack( RUN_ACTION ); return; }

			setImageRowsValue( MainApp.getMainFrame().getCameraPanel().getRowLabelValue() );
			setImageColsValue( MainApp.getMainFrame().getCameraPanel().getColLabelValue() );

			m_colSpinner.setValue( ( new Long( lCoords[ POINT_COL ] ) ).intValue() );
		}
		catch ( Throwable e )
		{
			MainApp.error( e );
		}
	}

	@Override
	public void setAutoRange()
	{
		m_xyPlot.getDomainAxis().setAutoRange( true );
		m_xyPlot.getRangeAxis().setAutoRange( true );
	}

	@Override
	public void run()
	{
		Thread.currentThread().setName( "Owl - ColPlotFrame" );
	
		m_runButton.setColored( ABORT_ACTION );
	
		try
		{
			synchronized( m_syncLock )
			{
				Point2D.Float[] points = null;

				if ( m_bStop )
				{
					throw new Exception( "Plot update aborted! You should not see this!" );
				}

				points = ArcImageJAPI.GetCol( ArcDeviceJAPI.GetCommonBufferVA(),
											  m_colSpinner.getInt(),
											  m_rowStartSpinner.getInt(),
											  m_rowEndSpinner.getInt(),
											  super.getImageRowsValue(),
											  super.getImageColsValue() );

				m_xyPlot.setPlot( points, "Col " + m_colSpinner.getInt() );

				if ( m_bStop )
				{
					throw new Exception( "Plot update aborted! You should not see this!" );
				}

				super.getStats( m_rowStartSpinner.getInt(),
								m_rowEndSpinner.getInt(),
								m_colSpinner.getInt(),
								m_colSpinner.getInt(),
								super.getImageRowsValue(),
								super.getImageColsValue() );
			}	// end synchronized code
		}
		catch ( Throwable e )
		{
			if ( !m_bStop )
			{
				MainApp.error( e.getMessage() );
			}
		}
		finally
		{
			m_runButton.setBlack( RUN_ACTION );
		}

		super.pack();
	}

	// +-----------------------------------------------------------------------------------+
	// | createCtrlPanel                                                                   |
	// +-----------------------------------------------------------------------------------+
	// | Creates the plot control ( input ) panel components.                              |
	// +-----------------------------------------------------------------------------------+
	private JPanel createCtrlPanel()
	{
		JPanel cPanel = new JPanel();
		cPanel.setLayout( new FlowLayout( FlowLayout.CENTER ) );
		cPanel.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.RAISED ) );

		m_colSpinner = new OwlNumberSpinner( 6 );
		m_colSpinner.addChangeListener( this );

		cPanel.add( new JLabel( "<html><b>COL</b></html>" ) );
		cPanel.add( javax.swing.Box.createHorizontalStrut( 5 ) );
		cPanel.add( m_colSpinner );
		cPanel.add( javax.swing.Box.createHorizontalStrut( 5 ) );

		JPanel rPanel = new JPanel();
		rPanel.setLayout( new FlowLayout( FlowLayout.CENTER ) );
		rPanel.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.RAISED ) );

		m_rowStartSpinner = new OwlNumberSpinner( 6 );
		m_rowStartSpinner.addChangeListener( this );

		m_rowEndSpinner   = new OwlNumberSpinner( 6 );
		m_rowEndSpinner.addChangeListener( this );

		rPanel.add( new JLabel( "<html><b>ROW</b></html>" ) );
		rPanel.add( javax.swing.Box.createHorizontalStrut( 5 ) );
		rPanel.add( new JLabel( "start: " ) );
		rPanel.add( m_rowStartSpinner );
		rPanel.add( javax.swing.Box.createHorizontalStrut( 5 ) );
		rPanel.add( new JLabel( "end: " ) );
		rPanel.add( m_rowEndSpinner );

		JPanel sPanel = createStatsPanel( true, this );

		JPanel panel = new JPanel();
		panel.setLayout( super.gbl );
		addComponent( panel, rPanel,              0, 0, 0, 0, 0, 0, 1, 1 );
		addComponent( panel, cPanel,              0, 0, 0, 0, 0, 1, 1, 1 );
		addComponent( panel, getCoordsSrcPanel(), 0, 0, 0, 0, 0, 2, 1, 1 );
		addComponent( panel, sPanel,              0, 0, 0, 0, 1, 0, 1, 3 );

		return panel;
	}

	// +-----------------------------------------------------------------------------------+
	// | enableChangeListeners                                                             |
	// +-----------------------------------------------------------------------------------+
	// | Enables/Disables the change listeners for the JSpinner components in this window. |
	// | Used to disable spinner change listeners at startup.                              |
	// +-----------------------------------------------------------------------------------+
	private void enableChangeListeners( boolean bEnable )
	{
		m_bChangesEnabled = bEnable;
	}
}
