package owl.main.dialogs;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.Border;

import owl.gui.utils.OwlUtilities;
import owl.main.MainApp;
import owl.main.owltypes.OwlDialog;
import owl.main.owltypes.OwlFrame;



public class AboutDialog extends OwlDialog implements ActionListener
{
	private static final long serialVersionUID = 3079393592115641130L;


	// +-------------------------------------------------------------------------------------------+
	// |  Constructor                                                                              |
	// +-------------------------------------------------------------------------------------------+
	public AboutDialog( OwlFrame owner )
    {
        super( owner, "About" );

        setUndecorated( true );

		Border jLineBorder  = BorderFactory.createLineBorder( Color.black, 4 );
		Border jEmptyBorder = BorderFactory.createEmptyBorder( 25, 25, 25, 25 );

		JPanel jPanel = new JPanel();
		jPanel.setBorder( BorderFactory.createCompoundBorder( jLineBorder, jEmptyBorder ) );
		jPanel.setLayout( new BoxLayout( jPanel, BoxLayout.Y_AXIS ) );
		jPanel.setBackground( new Color( 0xD1, 0xF0, 0xFF ) );
		add( jPanel );

		//  Create and add text stuff
        // +--------------------------------------------------------+
 		Component[] jComps = { createLabel( "Astronomical Research Cameras, Inc.", 18 ),
 							   createLabel( "Owl Image Aquisition Software", 18 ),
 							   createLabel( "Version " + MainApp.VERSION, 18 ),
							   Box.createVerticalStrut( 8 ),
							   new JLabel( new ImageIcon( MainApp.getBitmapPath() + "owl35.gif" ) ),
							   Box.createVerticalStrut( 8 ),
							   createLabel( "Scott Streit", 16 ),
							   createLabel( "streit@astro-cam.com", 16 ) };

		for ( Component jComp : jComps )
		{
			( ( JComponent )jComp ).setAlignmentX( Component.CENTER_ALIGNMENT );

			jPanel.add( jComp );
		}

		//  Create and add close button
        // +--------------------------------------------------------+
        ImageIcon jCloseIcon =
        		new ImageIcon( MainApp.getBitmapPath() + "Off.gif" );


        JButton jCloseButton = new JButton( "Close", jCloseIcon );
        jCloseButton.setAlignmentX( Component.CENTER_ALIGNMENT );
        jCloseButton.setContentAreaFilled( false );
        jCloseButton.setFocusPainted( false );
        jCloseButton.setBackground( Color.WHITE );
		jCloseButton.addActionListener( this );

		jPanel.add( Box.createVerticalStrut( 20 ) );
		jPanel.add( jCloseButton );

		//  Pack and center the frame
        // +--------------------------------------------------------+
		pack();

		OwlUtilities.centerFrame( this );
    }

	// +-------------------------------------------------------------------------------------------+
	// |  ActionListener overrides                                                                 |
	// +-------------------------------------------------------------------------------------------+
	public void actionPerformed( ActionEvent ae )
	{
		this.dispose();
	}

	// +-------------------------------------------------------------------------------------------+
	// |  createLabel                                                                              |
	// +-------------------------------------------------------------------------------------------+
	// |  Create a JLabel with the specified text and font size.                                   |
	// |                                                                                           |
	// |  @param sText - Label text                                                                |
	// |  @param dFontSize - Label text font size                                                  |
	// +-------------------------------------------------------------------------------------------+
	private JLabel createLabel( String sText, int dFontSize )
	{
		JLabel jLabel = new JLabel( sText );

		jLabel.setFont( new Font( Font.SANS_SERIF, Font.BOLD, dFontSize ) );

		return jLabel;
	}
}
