package owl.main.libs;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.io.FileFilter;
import java.util.Arrays;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JToolBar;

import owl.gui.utils.OwlButtonFactory;
import owl.gui.utils.OwlUtilities;
import owl.main.MainApp;
import owl.main.owltypes.OwlDirChooser;
import owl.main.owltypes.OwlFileChooser;
import owl.main.owltypes.OwlFrame;
import owl.main.owltypes.OwlIconedFileChooser;


public class JarBuilder extends OwlFrame implements ActionListener, ItemListener
{
	private static final long serialVersionUID = 2086058364849135749L;

	private final String CREATE_JAR_ACTION	= "CREATE";
	private final String CLASS_ADD_ACTION	= "CLASS ADD";
	private final String CLASS_DEL_ACTION	= "CLASS DEL";
	private final String PKG_ADD_ACTION		= "PKG ADD";
	private final String LIB_ADD_ACTION		= "LIB ADD";
	private final String LIB_DEL_ACTION		= "LIB DEL";

	private final String CLZ_PATH_PREF		= "ClassFilePath";
	private final String PKG_PATH_PREF		= "PkgFilePath";
	private final String JAR_PATH_PREF		= "JarFilePath";
	private final String LIB_PATH_PREF		= "LibFilePath";


	private JCheckBox		m_dllChkBox;
	private JPanel			m_libPanel;
	private Vector<String>	m_classVec;
	private Vector<PkgInfo>	m_pkgVec;
	private Vector<File>	m_libVec;
	private String			m_classFilePath;
	private String			m_pkgFilePath;
	private String			m_jarFilePath;
	private String			m_libFilePath;
	private JList			m_classList;
	private JList			m_libList;


	public class PkgInfo extends Object
	{
		public String sPkgName;
		public String sPkgPath;
		public String sClassName;

		public PkgInfo( String pkgName, String pkgPath )
		{
			sPkgName   = pkgName;
			sPkgPath   = pkgPath;
			sClassName = ( String )null;
		}
	}


	public JarBuilder()
	{
		super( "JAR Builder", true );

		super.setIconImage(
				( new ImageIcon(
						MainApp.getBitmapPath() + "jar.gif" ) ).getImage() );

		JPanel panel = new JPanel();
		panel.setLayout( gbl );

		addComponent( panel, createClassPanel(), 0, 0, 0, 0, 0, 0, 1, 1 );
		addComponent( panel, createLibPanel(),   0, 0, 0, 0, 1, 0, 1, 1 );

		JToolBar toolbar = new JToolBar();
		toolbar.add( super.createBoldToolbarButton( CREATE_JAR_ACTION, Color.black ) );
		super.appendToolbar( toolbar );

		addComponent( toolbar, super.TOOLBAR_INDEX );
		addComponent( panel,   super.CENTER_CONTAINER_INDEX );

		pack();

		OwlUtilities.centerFrame( this );

		m_classVec		= new Vector<String>();
		m_pkgVec		= new Vector<PkgInfo>();
		m_libVec		= new Vector<File>();

		m_classFilePath	= System.getProperty( "user.dir" );
		m_pkgFilePath	= System.getProperty( "user.dir" );
		m_jarFilePath	= System.getProperty( "user.dir" );
		m_libFilePath	= System.getProperty( "user.dir" );

		loadPreferences();
	}

	@Override
	public void itemStateChanged( ItemEvent ie )
	{
		m_libPanel.setVisible(  m_dllChkBox.isSelected() );

		pack();
	}

	public void actionPerformed( ActionEvent e )
	{
		// +---------------------------------------------------+
		// | CREATE JAR FILE
		// +---------------------------------------------------+
		if ( e.getActionCommand().equals( CREATE_JAR_ACTION ) )
		{
			OwlFileChooser fileChooser =
						new OwlFileChooser( new File( m_jarFilePath ),
											"jar" );

			if ( fileChooser.saveDialog() )
			{
				try
				{
					createJar( fileChooser.getSelectedFile() );

					m_classVec.clear();
					m_classList.setListData( m_classVec );

					m_libVec.clear();
					m_libList.setListData( m_libVec );

					m_dllChkBox.doClick();

					MainApp.info( "JAR successfully built!" );
				}
				catch ( Throwable ex )
				{
					MainApp.error( ex.getMessage() );
				}
			}
		}

		// +---------------------------------------------------+
		// | ADD CLASS(ES)
		// +---------------------------------------------------+
		else if ( e.getActionCommand().equals( CLASS_ADD_ACTION ) )
		{
			OwlIconedFileChooser fileChooser
					= new OwlIconedFileChooser(
							  new File( m_classFilePath ),
							  new String[] { "class" },
							  new String[] { "Java Class ( *.class )" },
							  new ImageIcon[] { new ImageIcon( MainApp.getBitmapPath() +
									  						   "class.gif" ) } );

			fileChooser.setMultiSelectionEnabled( true );

			if ( fileChooser.openDialog() )
			{
				File[] files = fileChooser.getSelectedFiles();

				if ( files != null )
				{
					m_classFilePath = files[ 0 ].getParent();

					for ( int i=0; i<files.length; i++ )
					{
						m_classVec.add( files[ i ].getAbsolutePath() );
					}
	
					m_classList.setListData( m_classVec );
				}
			}
		}

		// +---------------------------------------------------+
		// | ADD PACKAGE
		// +---------------------------------------------------+
		else if ( e.getActionCommand().equals( PKG_ADD_ACTION ) )
		{
			OwlDirChooser dirChooser =
						new OwlDirChooser( new File( m_pkgFilePath ) );

			if ( dirChooser.openDialog() )
			{
				File pkgDir = dirChooser.getSelectedFile();

				if ( pkgDir != null )
				{
					m_pkgFilePath = pkgDir.getPath();

					readPackageDir( pkgDir );
				}
			}
		}

		// +---------------------------------------------------+
		// | DELETE CLASS(ES)/PACKAGE ( from list )
		// +---------------------------------------------------+
		else if ( e.getActionCommand().equals( CLASS_DEL_ACTION ) )
		{
			m_classVec.removeAll( Arrays.asList( m_classList.getSelectedValues() ) );
			m_classList.setListData( m_classVec );
		}

		// +---------------------------------------------------+
		// | ADD LIB(S)
		// +---------------------------------------------------+
		else if ( e.getActionCommand().equals( LIB_ADD_ACTION ) )
		{
			OwlIconedFileChooser fileChooser
						= new OwlIconedFileChooser(
								  new File( m_libFilePath ),
								  new String[] { "dll", "jar" },
								  new String[] { "Native Library ( *.dll )", "Java Archive ( *.jar )" },
								  new ImageIcon[] { new ImageIcon( MainApp.getBitmapPath() + "libs.gif" ),
									  				new ImageIcon( MainApp.getBitmapPath() + "jar.gif" ) } );

			if ( !System.getProperty( "os.name" ).toLowerCase().contains( "win" ) )
			{
				fileChooser = new OwlIconedFileChooser(
							  new File( m_libFilePath ),
						  	  new String[] { "so", "jar" },
						  	  new String[] { "Native Library ( *.so )", "Java Archive ( *.jar )" },
						  	  new ImageIcon[] { new ImageIcon( MainApp.getBitmapPath() + "libs.gif" ),
								  				new ImageIcon( MainApp.getBitmapPath() + "jar.gif" ) } );
			}

			fileChooser.setMultiSelectionEnabled( true );

			if ( fileChooser.openDialog() )
			{
				File[] files = fileChooser.getSelectedFiles();

				if ( files != null )
				{
					m_libFilePath = files[ 0 ].getParent();
	
					for ( int i=0; i<files.length; i++ )
					{
						if ( !m_libVec.contains( files[ i ] ) )
						{
							m_libVec.add( files[ i ] );
						}
					}
	
					m_libList.setListData( m_libVec );
				}
			}
		}

		// +---------------------------------------------------+
		// | DELETE LIB(S) ( from list )
		// +---------------------------------------------------+
		else if ( e.getActionCommand().equals( LIB_DEL_ACTION ) )
		{
			m_libVec.removeAll( Arrays.asList( m_libList.getSelectedValues() ) );
			m_libList.setListData( m_libVec );
		}

		// +---------------------------------------------------+
		// | PASS TO SUPER
		// +---------------------------------------------------+
		else
		{
			super.actionPerformed( e );
		}
	}

	private void createJar( File jarFile ) throws Throwable
	{
		Vector<String> cmdVec = new Vector<String>();

		if ( jarFile == null )
		{
			throw new Exception( "Invalid JAR file parameter: null!" );
		}

		if ( !jarFile.getAbsolutePath().contains( ".jar" ) )
		{
			jarFile = new File( jarFile.getAbsolutePath() + ".jar" );
		}

		m_jarFilePath = jarFile.getParent();
			
		if ( jarFile.exists() )
		{
			if ( !jarFile.delete() )
			{
				throw new Exception(
							"Failed to delete existing file: " + jarFile );
			}
		}

		cmdVec.add( "jar" );
		cmdVec.add( "cf" );
		cmdVec.add( jarFile.getPath() );

		if ( !m_libVec.isEmpty() )
		{
			for ( int i=0; i<m_libVec.size(); i++ )
			{
				cmdVec.add( "-C" );
				cmdVec.add( m_libVec.get( i ).getParent() );
				cmdVec.add( m_libVec.get( i ).getName() );
			}
		}


		if ( !m_pkgVec.isEmpty() )
		{
			PkgInfo pkgInfo = ( PkgInfo )null;
		
			for ( int i=0; i<m_pkgVec.size(); i++ )
			{
				pkgInfo = m_pkgVec.get( i );
		
				cmdVec.add( "-C" );
				cmdVec.add( pkgInfo.sPkgPath );
				cmdVec.add( pkgInfo.sClassName );
			}
		}
		
		if ( !m_classVec.isEmpty() )
		{
			for ( int i=0; i<m_classVec.size(); i++ )
			{
				if ( !( new File( m_classVec.get( i ) ) ).isDirectory() )
				{
					cmdVec.add( "-C" );
					cmdVec.add( OwlUtilities.getPath( m_classVec.get( i ) ) );

					cmdVec.add( m_classVec.get( i ).substring( m_classVec.get( i ).lastIndexOf(
								System.getProperty( "file.separator" ) ) + 1 ) );
				}
			}
		}

		Process proc =
			Runtime.getRuntime().exec( cmdVec.toArray( new String[ cmdVec.size() ] ) );

		proc.waitFor();
	}

	private void readPackageDir( File pkgDir )
	{
		readPackageDir( pkgDir, null );
	}

	private void readPackageDir( File pkgDir, String superPkgName )
	{
		String  pkgName = ( superPkgName == null ? pkgDir.getName() : ( superPkgName + "/" + pkgDir.getName() ) );
		String  pkgPath = ( superPkgName == null ? pkgDir.getParent() : getPkgParentPath( pkgName, pkgDir.getAbsolutePath() ) );
		PkgInfo pkgInfo = ( PkgInfo )null;

		FileFilter filter = new FileFilter()
		{
		    public boolean accept( File file )
		    {
				if ( file.isDirectory() )
				{
					return true;
				}

		        return file.getName().endsWith( ".class" );
		    }
		};

		File[] pkgFiles = pkgDir.listFiles( filter );

		for ( int i=0; i<pkgFiles.length; i++ )
		{
			if ( pkgFiles[ i ].isDirectory() )
			{
				readPackageDir( pkgFiles[ i ], pkgName );
				continue;
			}

			pkgInfo = new PkgInfo( pkgName, pkgPath );
			pkgInfo.sClassName = pkgName + "/" + pkgFiles[ i ].getName();

			m_pkgVec.add( pkgInfo );
		}

		if ( superPkgName == null )
		{
			m_classVec.add( pkgDir.getPath() );
			m_classList.setListData( m_classVec );
		}
	}

	private String getPkgParentPath( String pkgName, String pkgPath )
	{
		String retStr = null;

		String newName = pkgName.replace( "/", System.getProperty( "file.separator" ) );
		int pkgIndex = pkgPath.lastIndexOf( newName );	

		if ( pkgIndex >= 0 )
		{
			retStr = pkgPath.substring( 0, pkgIndex );
		}

		return retStr;
	}

	private JPanel createClassPanel()
	{
		JPanel panel = new JPanel();
		panel.setBorder( BorderFactory.createTitledBorder( "Class/Package List" ) );
		panel.setLayout( super.gbl );

		m_classList = new JList();
		JScrollPane classListPane = new JScrollPane( m_classList );
		Dimension dimen = classListPane.getPreferredSize();
		classListPane.setPreferredSize( new Dimension( dimen.width + 120, dimen.height ) );
		super.addComponent( panel, classListPane, 0, 0, 0, 0, 0, 0, 3, 1 );

		JButton pkgAddButton = OwlButtonFactory.createIconButton( "pkg.gif", 26, 26, "Add Package", PKG_ADD_ACTION, this );
		super.addComponent( panel, pkgAddButton, 0, 0, 0, 0, GridBagConstraints.NORTHWEST, 0, 1, 1, 1 );

		JButton classAddButton = OwlButtonFactory.createIconButton( "class.gif", 26, 26, "Add Class", CLASS_ADD_ACTION, this );
		super.addComponent( panel, classAddButton, 0, 0, 0, 0, GridBagConstraints.NORTHWEST, 1, 1, 1, 1 );

		JButton classDelButton = OwlButtonFactory.createIconButton( "Off.gif", 26, 26, "Remove File", CLASS_DEL_ACTION, this );
		super.addComponent( panel, classDelButton, 0, 0, 0, 0, GridBagConstraints.NORTHWEST, 2, 1, 1, 1 );

		return panel;
	}

	private JPanel createLibPanel()
	{
		m_dllChkBox = new JCheckBox( "Add Dependent Libraries" );
		m_dllChkBox.addItemListener( this );

		String sLibExt = ( !System.getProperty( "os.name" ).toLowerCase().contains( "win" ) ? ".so" : ".dll" );

		m_libPanel = new JPanel();
		m_libPanel.setBorder( BorderFactory.createTitledBorder( "Additional library List ( .jar, " + sLibExt + " )" ) );
		m_libPanel.setLayout( super.gbl );
		m_libPanel.setVisible( false );

		String sMsg = "<html><i><font color=\"#444444\">" +
					  "** Additional support libraries may be zipped with the new java archive." +
					  "</font></i></html>";
		addComponent( m_libPanel, new JLabel( sMsg ), 0, 0, 5, 0, GridBagConstraints.NORTHWEST, 0, 0, 1, 1 );

		m_libList = new JList();
		JScrollPane dllListPane = new JScrollPane( m_libList );
		Dimension dimen = dllListPane.getPreferredSize();
		dllListPane.setPreferredSize( new Dimension( dimen.width + 120, dimen.height ) );
		addComponent( m_libPanel, dllListPane, 0, 0, 0, 0, 1, 0, 2, 1 );


		JButton dllAddButton = OwlButtonFactory.createIconButton( "Plus2.gif", 26, 26, "Add File", LIB_ADD_ACTION, this );
		addComponent( m_libPanel, dllAddButton, 0, 0, 0, 0, GridBagConstraints.NORTHWEST, 1, 1, 1, 1 );

		JButton dllDelButton = OwlButtonFactory.createIconButton( "Off.gif", 26, 26, "Remove File", LIB_DEL_ACTION, this );
		addComponent( m_libPanel, dllDelButton, 0, 0, 0, 0, GridBagConstraints.NORTHWEST, 2, 1, 1, 1 );

		JPanel panel = new JPanel();
		panel.setLayout( super.gbl );

		addComponent( panel, m_dllChkBox, 0, 0, 0, 0, GridBagConstraints.NORTHWEST, 0, 0, 1, 1 );
		addComponent( panel, m_libPanel,  0, 0, 0, 0, GridBagConstraints.NORTHWEST, 2, 0, 1, 1 );

		return panel;
	}

	//--------------------------------------------------------------------------
    //   Protected Methods:
    //--------------------------------------------------------------------------
	protected void loadPreferences()
	{
		super.loadPreferences();

		try
		{
			String winName = super.getTitle();

			m_classFilePath = MainApp.getPreferences().get(
						winName + CLZ_PATH_PREF, System.getProperty( "user.dir" ) );

			m_pkgFilePath   = MainApp.getPreferences().get(
						winName + PKG_PATH_PREF, System.getProperty( "user.dir" ) );

			m_jarFilePath   = MainApp.getPreferences().get(
						winName + JAR_PATH_PREF, System.getProperty( "user.dir" ) );

			m_libFilePath   = MainApp.getPreferences().get(
						winName + LIB_PATH_PREF, System.getProperty( "user.dir" ) );
		}
		catch ( NullPointerException npe ) {}
	}

	protected void savePreferences()
	{
		super.savePreferences();

		try
		{
			String winName = super.getTitle();

			MainApp.getPreferences().put( winName + CLZ_PATH_PREF, m_classFilePath );
			MainApp.getPreferences().put( winName + PKG_PATH_PREF, m_pkgFilePath );
			MainApp.getPreferences().put( winName + JAR_PATH_PREF, m_jarFilePath );
			MainApp.getPreferences().put( winName + LIB_PATH_PREF, m_libFilePath );
		}
		catch ( Throwable e )
		{
			MainApp.error( e.toString() );
		}
	}
}
