package owl.main.libs;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.util.Arrays;
import java.util.Vector;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

import javax.swing.BorderFactory;
import javax.swing.DefaultListCellRenderer;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JToolBar;

import owl.gui.utils.OwlButtonFactory;
import owl.gui.utils.OwlUtilities;
import owl.main.MainApp;
import owl.main.owltypes.OwlFrame;
import owl.main.owltypes.OwlIconedFileChooser;
import owl.main.scripting.MainInterpreter;


public class JarLibFrame extends OwlFrame implements ActionListener
{
	private static final long serialVersionUID = -2010023075228835486L;

	private final String FILE_PATH_PREF		= "FilePath";
	private final String ADD_LIB_ACTION		= "AddLib";
	private final String DEL_LIB_ACTION		= "DelLib";
	private final String VALID_INI_KEY		= "[EXTERNAL JAR LIBS]";
	private final String INI_FILE			= "ExternLibs.ini";
	private final String EXTERN_LIB_PATH	= "ExternLibs";
	private final String LIST_SEPERATOR		= "-";

	private JList	m_jarList;
	private File	m_file;


	// +----------------------------------------------------------------------+
	// |  Constructor                                                         |
	// +----------------------------------------------------------------------+
	public JarLibFrame()
	{
		super( "External Libraries", false );

		setIconImage(
			( new ImageIcon(
					MainApp.getBitmapPath() + "libs.gif" ) ).getImage() );

		JToolBar toolbar = new JToolBar();
		appendToolbar( toolbar );

		addComponent( toolbar, super.TOOLBAR_INDEX );
		addComponent( new JScrollPane( createComponents() ), super.CENTER_CONTAINER_INDEX );

		OwlUtilities.centerFrame( this );

		pack();

		m_file = new File( System.getProperty( "user.dir" ) );

		loadPreferences();
	}

	// +----------------------------------------------------------------------+
	// |  STATIC - loadLibraries                                              |
	// +----------------------------------------------------------------------+
	public static void loadLibraries( boolean bShowMsg )
	{
		JarLibFrame frame = new JarLibFrame();

		try
		{
			Vector<String> vList = frame.getList();
	
			for ( int i=0; i<vList.size(); i++ )
			{
				frame.updatePaths( frame.convertListTextToFile( vList.get( i ).trim() ) );
			}

			if ( bShowMsg )
			{
				MainApp.info( "External archives SUCCESSFULLY loaded!" );
			}
		}
		catch ( Throwable e )
		{
			MainApp.error( e.getMessage() );
		}
	}

	// +----------------------------------------------------------------------+
	// |  actionPerformed                                                     |
	// +----------------------------------------------------------------------+
	// |  See ActionListener for details                                      |
	// +----------------------------------------------------------------------+
	@Override
	public void actionPerformed( ActionEvent ae )
	{
		//  Add Jar
		// +-------------------------------------------------+
		if ( ae.getActionCommand().equals( ADD_LIB_ACTION ) )
		{
			OwlIconedFileChooser fileChooser
					= new OwlIconedFileChooser( m_file,
					  new String[] { "jar" },
					  new String[] { "Java Archive ( *.jar )" },
					  new ImageIcon[] { new ImageIcon( MainApp.getBitmapPath() + "jar.gif" ) } );

			fileChooser.setFrameIcon( "jar.gif" );

			if ( fileChooser.openDialog() )
			{
				boolean bRemoveExisting = false;

				m_file = fileChooser.getSelectedFile();

				try
				{
					String sLibName = OwlUtilities.getFileNameNoExtension( m_file.getName() );

					if ( listContains( sLibName ) || libraryExists( sLibName ) )
					{
						boolean bIsWin = System.getProperty( "os.name" ).toLowerCase().contains( "windows" );

						String sMsg = "<html><font color=\"#B40404\">" + 
									  "A library with the same name already exists. " +
									  "<b>Replace existing files?</b></font>\n" +
									  "<html><font color=\"#444444\">" +
									  "<b>NOTE</b>: Libraries containing native files ( " +
									  ( bIsWin ? ".dll" : ".so" ) + " ) " +
									  "can only be removed before calling code executes!" +
									  "</font></html>";

						int dRetVal = JOptionPane.showConfirmDialog(
														null,
														sMsg,
														"Replace Library",
														JOptionPane.YES_NO_OPTION,
														JOptionPane.QUESTION_MESSAGE,
														new ImageIcon( MainApp.getBitmapPath() + "ErrMsg.gif" ) );

						if ( dRetVal == JOptionPane.YES_OPTION )
						{
							bRemoveExisting = true;
						}
						else
						{
							return;
						}
					}

					File fInstallFile = convertToInstallFile( m_file );

					if ( bRemoveExisting )
					{
						File[] fLibFiles = ( new File( fInstallFile.getParent() ) ).listFiles();

						for ( File fFile : fLibFiles )
						{
							fFile.delete();
						}

						( new File( fInstallFile.getParent() ) ).delete();

						removePath( fInstallFile );

						String[] sItems = { getListText( fInstallFile ) };
						delListItems( sItems );

						writeListToFile();
					}

					fInstallFile.getParentFile().mkdir();

					OwlUtilities.copyFile( m_file, fInstallFile );

					extractNativeLibs( fInstallFile );
					extractJARs( fInstallFile );

					addListItem( fInstallFile );
					updatePaths( fInstallFile );
					writeListToFile();

					MainApp.info( "External library SUCCESSFULLY loaded!" );
				}
				catch ( Throwable e )
				{
					MainApp.error( e.getMessage() );
				}
			}
		}

		//  Remove Jar
		// +-------------------------------------------------+
		else if ( ae.getActionCommand().equals( DEL_LIB_ACTION ) )
		{
			try
			{
				Object[] objArr = m_jarList.getSelectedValues();
	
				delListItems(
						Arrays.copyOf(
								objArr, objArr.length, String[].class ) );

				for ( Object fileObj : objArr )
				{
					File fLibFile = convertListTextToFile( ( String )fileObj );

					//  NOTE: The file/dir won't actually be deleted unless a new file
					//  is created. i.e. using fLibFile.getParentFile().listFiles()
					//  won't work. File won't be deleted!
					// +----------------------------------------------------------------+
					File[] fLibFiles = ( new File( fLibFile.getParent() ) ).listFiles();

					for ( File fFile : fLibFiles )
					{
						fFile.delete();
					}

					( new File( fLibFile.getParent() ) ).delete();

					removePath( fLibFile );
				}

				writeListToFile();
			}
			catch ( Throwable e )
			{
				MainApp.error( e.getMessage() );
			}
		}

		//  Pass to super
		// +-------------------------------------------------+
		else
		{
			super.actionPerformed( ae );
		}
	}

	// +----------------------------------------------------------------------+
	// |  updatePaths                                                         |
	// +----------------------------------------------------------------------+
	// |  Updates the JRE class and library paths along with the script       |
	// |  interpreter class path.                                             |
	// +----------------------------------------------------------------------+
	public void updatePaths( File file ) throws Throwable
	{
		//  Add classpath to script interpreter
		// +-------------------------------------------+
		MainInterpreter.addExternalClassPath( file.toURI().toURL() );
	}

	// +----------------------------------------------------------------------+
	// |  removePath                                                          |
	// +----------------------------------------------------------------------+
	// |  Updates the JRE class and library paths along with the script       |
	// |  interpreter class path.                                             |
	// +----------------------------------------------------------------------+
	public void removePath( File file ) throws Throwable
	{
		//  Remove classpath from script interpreter
		// +-------------------------------------------+
		MainInterpreter.removeExternalClassPath( file.toURI().toURL() );
	}

	// +----------------------------------------------------------------------+
	// |  libraryExists                                                       |
	// +----------------------------------------------------------------------+
	// |  Returns 'true' if the specified library folder exists in the apps   |
	// |  external library folder.  Returns 'false' otherwise.                |
	// |                                                                      |
	// |  @param sLibName - The library folder name to search for.            |
	// +----------------------------------------------------------------------+
	public boolean libraryExists( String sLibName )
	{
		if ( sLibName != null && !sLibName.isEmpty() )
		{
			File fSearchPath = new File(
									MainApp.getAppPath() + EXTERN_LIB_PATH );

			File[] fLibFiles = fSearchPath.listFiles();
	
			if ( fLibFiles != null )
			{
				for ( File fFile : fLibFiles )
				{
					if ( fFile.getAbsolutePath().contains( sLibName ) )
					{
						return true;
					}
				}
			}
		}

		return false;
	}

	// +----------------------------------------------------------------------+
	// |  extractNativeLibs                                                   |
	// +----------------------------------------------------------------------+
	// |  Extracts all native libraries ( .dll/.so ) from the specified jar   |
	// |  file.  The files are extracted to the same directory as the jar     |
	// |  file.                                                               |
	// +----------------------------------------------------------------------+
	public void extractNativeLibs( File file ) throws Throwable
	{
		try
		{
		    ZipInputStream in = new ZipInputStream(
		    						new FileInputStream( file.getPath() ) );

		    ZipEntry zipEntry = null;

		    while ( ( zipEntry = in.getNextEntry() ) != null )
		    {
		    	String sEntryName = zipEntry.getName();

		    	if ( sEntryName.contains( ".dll" ) || sEntryName.contains( ".so" ) )
		    	{
		    		String outFilename =
		    				OwlUtilities.getPath( file.getPath() ) +
		    							 ( new File( sEntryName ) ).getName();

		    		OutputStream out = new FileOutputStream( outFilename );

				    // Transfer bytes from the ZIP file to the output file
				    byte[] buf = new byte[ 1024 ];
				    int    len = 0;

				    while ( ( len = in.read( buf ) ) > 0 )
				    {
				    	out.write( buf, 0, len );
				    }
	
				    out.close();

				    MainApp.info( "Extracted native library -> " + sEntryName );
		    	}
		    }

		    in.close();
		}
		catch ( Throwable e )
		{
			MainApp.error( e.getMessage() );
		}
	}

	// +----------------------------------------------------------------------+
	// |  extractJARs                                                         |
	// +----------------------------------------------------------------------+
	// |  Extracts all java libraries ( .jar ) from the specified jar file.   |
	// |  The files are extracted to the same directory as the specified      |
	// |  file.                                                               |
	// +----------------------------------------------------------------------+
	public void extractJARs( File file ) throws Throwable
	{
		try
		{
		    ZipInputStream in = new ZipInputStream(
		    						new FileInputStream( file.getPath() ) );

		    ZipEntry zipEntry = null;

		    while ( ( zipEntry = in.getNextEntry() ) != null )
		    {
		    	String sEntryName = zipEntry.getName();

		    	if ( sEntryName.contains( ".jar" ) )
		    	{
		    		String outFilename =
		    				OwlUtilities.getPath( file.getPath() ) +
		    							 ( new File( sEntryName ) ).getName();

		    		OutputStream out = new FileOutputStream( outFilename );

				    // Transfer bytes from the ZIP file to the output file
				    byte[] buf = new byte[ 1024 ];
				    int    len = 0;

				    while ( ( len = in.read( buf ) ) > 0 )
				    {
				    	out.write( buf, 0, len );
				    }
	
				    out.close();

				    MainApp.info( "Extracted java library -> " + sEntryName );
		    	}
		    }

		    in.close();
		}
		catch ( Throwable e )
		{
			MainApp.error( e.getMessage() );
		}
	}

	// +----------------------------------------------------------------------+
	// |  addListItem                                                         |
	// +----------------------------------------------------------------------+
	// |  Adds a new item to the JList.                                       |
	// |                                                                      |
	// |  @param sItems - The file to add to the list.                        |
	// +----------------------------------------------------------------------+
	public void addListItem( File file ) throws Exception
	{
		Vector<String> vList = new Vector<String>();

		if ( file == null )
		{
			throw new Exception( "Invalid file parameter: null" );
		}

		String sText = getListText( file );

		vList.add( sText );

		for ( int i=0; i<m_jarList.getModel().getSize(); i++ )
		{
			//
			//  Prevent duplicate values
			//
			if ( sText.equals( ( String )m_jarList.getModel().getElementAt( i ) ) )
			{
				throw new Exception( "Library already loaded into system!" );
			}

			vList.add( ( String )m_jarList.getModel().getElementAt( i ) );
		}

		m_jarList.setListData( vList );
	}

	// +----------------------------------------------------------------------+
	// |  delListItems                                                        |
	// +----------------------------------------------------------------------+
	// |  Removes all specified items from the JList.                         |
	// |                                                                      |
	// |  @param sItems - The array of items to remove.                       |
	// +----------------------------------------------------------------------+
	public void delListItems( String[] sItems )
	{
		Vector<String> vList = getList();

		if ( sItems != null )
		{
			for ( int i=0; i<sItems.length; i++ )
			{
				vList.remove( sItems[ i ] );
			}

			m_jarList.setListData( vList );
		}
	}

	// +----------------------------------------------------------------------+
	// |  listContains                                                        |
	// +----------------------------------------------------------------------+
	// |  Returns 'true' if the JList contains the specified text. Returns    |
	// |  'false' otherwise.                                                  |
	// |                                                                      |
	// |  @param sText - The text to search for.                              |
	// +----------------------------------------------------------------------+
	public boolean listContains( String sText )
	{
		Vector<String> vList = getList();

		if ( sText != null && !sText.isEmpty() )
		{
			for ( String sListText : vList )
			{
				if ( sListText.contains( sText ) )
				{
					return true;
				}
			}
		}

		return false;
	}

	// +----------------------------------------------------------------------+
	// |  getList                                                             |
	// +----------------------------------------------------------------------+
	// |  Returns the JList as a Vector<String>.                              |
	// +----------------------------------------------------------------------+
	public Vector<String> getList()
	{
		Vector<String> vList = new Vector<String>();

		for ( int i=0; i<m_jarList.getModel().getSize(); i++ )
		{
			vList.add( ( String )m_jarList.getModel().getElementAt( i ) );
		}

		return vList;
	}

	// +----------------------------------------------------------------------+
	// |  readListFromFile                                                    |
	// +----------------------------------------------------------------------+
	// |  Reads the library list from the backing file.                       |
	// +----------------------------------------------------------------------+
	public Vector<String> readListFromFile() throws Throwable
	{
		Vector<String>	vList	= new Vector<String>();
		BufferedReader	br		= null;
		String			sLine	= "";

		try
		{
			br = new BufferedReader( new FileReader( INI_FILE ) );
	
			while ( br.ready() )
			{
				sLine = br.readLine();
	
				if ( !sLine.contains( VALID_INI_KEY ) )
				{
					vList.add( sLine );
				}
			}
		}
		catch ( Throwable e )
		{
			if ( br != null ) { br.close(); }

			throw e;
		}

		if ( br != null ) { br.close(); }

		return vList;
	}

	// +----------------------------------------------------------------------+
	// |  writeList                                                           |
	// +----------------------------------------------------------------------+
	// |  Writes the JList to the backing file.                               |
	// +----------------------------------------------------------------------+
	public void writeListToFile() throws Throwable
	{
		FileWriter     fw    = null;
		PrintWriter    pw    = null;

		try
		{
			fw = new FileWriter( INI_FILE, false );
			pw = new PrintWriter( fw, true );
	
			pw.println( VALID_INI_KEY );
	
			for ( int i=0; i<m_jarList.getModel().getSize(); i++ )
			{
				pw.println( m_jarList.getModel().getElementAt( i ) );
			}
		}
		catch ( Throwable e )
		{
			if ( pw != null ) { pw.close(); }
			if ( fw != null ) { fw.close(); }

			throw e;
		}

		if ( pw != null ) { pw.close(); }
		if ( fw != null ) { fw.close(); }
	}

	//--------------------------------------------------------------------------
    //   Protected Methods:
    //--------------------------------------------------------------------------
	protected void loadPreferences()
	{
		super.loadPreferences();

		try
		{
			m_file = new File( MainApp.getPreferences().get(
										super.getTitle() + FILE_PATH_PREF,
										System.getProperty( "user.dir" ) ) );
		}
		catch ( NullPointerException npe ) {}
	}

	protected void savePreferences()
	{
		super.savePreferences();

		try
		{
			MainApp.getPreferences().put( super.getTitle() + FILE_PATH_PREF,
										  m_file.getPath() );
		}
		catch ( Throwable e )
		{
			MainApp.error( e.toString() );
		}
	}

	// +----------------------------------------------------------------------+
	// |  createComponents                                                    |
	// +----------------------------------------------------------------------+
	// |  Creates the window components.                                      |
	// +----------------------------------------------------------------------+
	private JPanel createComponents()
	{
		JPanel listPanel = new JPanel();
		listPanel.setBorder( BorderFactory.createTitledBorder( "External Library List" ) );
		listPanel.setLayout( super.gbl );

		m_jarList = new JList();
		m_jarList.setCellRenderer( new IconCellRenderer() );

		JScrollPane jarListPane = new JScrollPane( m_jarList );
		Dimension dimen = jarListPane.getPreferredSize();
		jarListPane.setPreferredSize( new Dimension( dimen.width + 180, dimen.height ) );

		JButton jarAddButton = OwlButtonFactory.createIconButton( "Plus2.gif", 26, 26,
																  "Add Library", ADD_LIB_ACTION,
																   this );

		JButton jarDelButton = OwlButtonFactory.createIconButton( "Off.gif", 26, 26,
																  "Remove Library", DEL_LIB_ACTION,
																   this );

		addComponent( listPanel, jarListPane,   0, 0, 0, 0, 1, 0, 3, 1 );
		addComponent( listPanel, jarAddButton,  0, 0, 0, 0, GridBagConstraints.NORTHWEST, 1, 1, 1, 1 );
		addComponent( listPanel, jarDelButton,  0, 0, 0, 0, GridBagConstraints.NORTHWEST, 3, 1, 1, 1 );

		try
		{
			Vector<String> vList = readListFromFile();
			m_jarList.setListData( vList );
		}
		catch( Throwable e ) {}

		JPanel panel = new JPanel();
		panel.setLayout( super.gbl );

		addComponent( panel, listPanel, 5, 5, 5, 5, 0, 0, 1, 1 );

		return panel;
	}

	// +----------------------------------------------------------------------+
	// |  convertToInstallFile                                                |
	// +----------------------------------------------------------------------+
	// |  Converts the user supplied filename to contain a new path located   |
	// |  within the JarLibFrame.EXTERN_LIB_PATH.  The filename is used as a  |
	// |  sub-directory in the JarLibFrame.EXTERN_LIB_PATH folder.            |
	// |                                                                      |
	// |  For example:  C:\xxx\yyy\MyLib.jar  becomes                         |
	// |                JarLibFrame.EXTERN_LIB_PATH\MyLib\MyLib.jar           |
	// |                                                                      |
	// |  @param fFile - The full filename ( + path ) of the user library.    |
	// +----------------------------------------------------------------------+
	private File convertToInstallFile( File fFile ) throws Exception
	{
		String sTokens[] = null;

		if ( fFile.getName().contains( ".jar" ) )
		{
			sTokens = fFile.getName().split( ".jar" );
		}

		if ( sTokens == null || sTokens.length != 1 )
		{
			throw new Exception( "Failed to parse filename: " +
								 fFile.getAbsolutePath() +
								 "\nExpected format: [ C:\\xxx\\my.jar ]" );
		}

		return ( new File( MainApp.getAppPath() +
							EXTERN_LIB_PATH +
							File.separatorChar +
							sTokens[ 0 ] +
							File.separatorChar +
							fFile.getName() ) );
	}

	// +----------------------------------------------------------------------+
	// |  convertListTextToFile                                               |
	// +----------------------------------------------------------------------+
	// |  Converts the supplied library list text to a full path filename.    |
	// |                                                                      |
	// |  For example:  MyLib.jar - JarLibFrame.EXTERN_LIB_PATH\MyLib         |
	// |                becomes                                               |
	// |                JarLibFrame.EXTERN_LIB_PATH\MyLib\MyLib.jar           |
	// |                                                                      |
	// |  @param sListText - The text from the list.                          |
	// +----------------------------------------------------------------------+
	private File convertListTextToFile( String sListText ) throws Exception
	{
		String sTokens[] = sListText.split( LIST_SEPERATOR );

		if ( sTokens.length != 2 )
		{
			throw new Exception( "Failed to parse list text: " +
								 sListText +
								 "\nExpected format: [ libName  - libPath ]" );
		}

		return ( new File( sTokens[ 1 ].trim() +
						   File.separatorChar  +
						   sTokens[ 0 ].trim() ) );
	}

	// +----------------------------------------------------------------------+
	// |  getListText                                                         |
	// +----------------------------------------------------------------------+
	// |  Converts the supplied full path library filename to list text       |
	// |  format.                                                             |
	// |                                                                      |
	// |  For example:  JarLibFrame.EXTERN_LIB_PATH\MyLib\MyLib.jar           |
	// |                becomes                                               |
	// |                MyLib.jar - JarLibFrame.EXTERN_LIB_PATH\MyL           |
	// |                                                                      |
	// |  @param fFile - The file to convert to list text format.             |
	// +----------------------------------------------------------------------+
	private String getListText( File fFile )
	{
		return ( fFile.getName() +
				 "  " + LIST_SEPERATOR + " " +
				 fFile.getParent() );
	}

	// +----------------------------------------------------------------------+
	// |  Class IconCellRenderer                                              |
	// +----------------------------------------------------------------------+
	// |  The icon cell renderer for the JList component.                     |
	// +----------------------------------------------------------------------+
	private class IconCellRenderer extends DefaultListCellRenderer
	{
		private static final long serialVersionUID = 6244055527295675685L;

		final ImageIcon JAR_ICON = new ImageIcon( MainApp.getBitmapPath() + "loadJar.gif" );

	    public Component getListCellRendererComponent( JList list, Object value, int index,	boolean isSelected, boolean bHasFocus )
	    {
	    	// The DefaultListCellRenderer class will take care of
	    	// the JLabels text property, it's foreground and background
	    	//colors, and so on.
	    	super.getListCellRendererComponent( list, value, index, isSelected, bHasFocus );

	    	setIcon( JAR_ICON );

	    	return this;
	    }
	}
}
