package owl.main.owltypes;

import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;


// +--------------------------------------------------------------------------------------------+
// |  OwlNumberSpinner Class                                                                    |
// +--------------------------------------------------------------------------------------------+
// |  Extends javax.swing.JSpinner with the editor as a javax.swing.JTextField and the Model as |
// |  a javax.swing.SpinnerNumberModel.  Also supports focus highlighting.                      |
// +--------------------------------------------------------------------------------------------+
public class OwlNumberSpinner extends JSpinner
{
	private static final long serialVersionUID = -3220245553963322290L;


	// +----------------------------------------------------------------------------------------+
	// |  Constructor                                                                           |
	// +----------------------------------------------------------------------------------------+
	public OwlNumberSpinner( int dValue, int dMinimum, int dMaximum, int dStepSize, int dColumns )
	{
		super();

		setModel( new SpinnerNumberModel( dValue, dMinimum, dMaximum, dStepSize ) );

		setEditor( new OwlSpinnerEditor( this, dColumns ) );
	}

	// +----------------------------------------------------------------------------------------+
	// |  Constructor                                                                           |
	// +----------------------------------------------------------------------------------------+
	public OwlNumberSpinner( int dInitVal, int dColumns )
	{
		this( dInitVal, 0, ( int )Math.pow( 2, 32 ), 1, dColumns );
	}

	// +----------------------------------------------------------------------------------------+
	// |  Constructor                                                                           |
	// +----------------------------------------------------------------------------------------+
	public OwlNumberSpinner( int dColumns )
	{
		this( 0, 0, ( int )Math.pow( 2, 32 ), 1, dColumns );
	}

	// +----------------------------------------------------------------------------------------+
	// |  getInt                                                                                |
	// +----------------------------------------------------------------------------------------+
	public int getInt()
	{
		return ( ( Integer )getValue() ).intValue();
	}

	// +----------------------------------------------------------------------------------------+
	// |  setMaximum                                                                            |
	// +----------------------------------------------------------------------------------------+
	public void setMaximum( int dMaxVal )
	{
		if ( getInt() > dMaxVal )
		{
			setValue( new Integer( dMaxVal ) );
		}

		( ( SpinnerNumberModel )getModel() ).setMaximum( Integer.valueOf( dMaxVal ) );
	}

	// +----------------------------------------------------------------------------------------+
	// |  getMaximum                                                                            |
	// +----------------------------------------------------------------------------------------+
	public int getMaximum()
	{
		return ( ( Integer )( ( SpinnerNumberModel )getModel() ).getMaximum() ).intValue();
	}


	// +----------------------------------------------------------------------------------------+
	// |  OwlSpinnerEditor Class                                                                |
	// +----------------------------------------------------------------------------------------+
	// |  Provides a javax.swing.JTextField that handles change events from a JSpinner class.   |
	// |  Focus and caret events are also handled to keep the data properly updated.            |
	// +----------------------------------------------------------------------------------------+
	protected class OwlSpinnerEditor extends JTextField implements ChangeListener, CaretListener, FocusListener
	{
		private static final long serialVersionUID = 1422767254542733183L;

		private JSpinner m_jSpinner;

		// +------------------------------------------------------------------------------------+
		// |  Constructor                                                                       |
		// +------------------------------------------------------------------------------------+
		public OwlSpinnerEditor( JSpinner jSpinner, int dColumns )
		{
			super( "0", dColumns );

			m_jSpinner = jSpinner;

			m_jSpinner.addChangeListener( this );

			addFocusListener( this );
			addCaretListener( this );

			setHorizontalAlignment( JTextField.LEFT );
		}

		// +------------------------------------------------------------------------------------+
		// |  stateChanged - ChangeListener method                                              |
		// +------------------------------------------------------------------------------------+
		@Override
		public void stateChanged( ChangeEvent evt )
		{
			if ( evt.getSource() instanceof javax.swing.JSpinner )
			{
				JSpinner jSpinner = ( JSpinner )evt.getSource();

				if ( jSpinner.getValue() instanceof java.lang.Integer )
				{
					Integer value = ( Integer )jSpinner.getValue();
		
					if ( value.compareTo( Integer.valueOf( getText() ) ) != 0 )
					{
						setText( value.toString() );
					}
				}
			}
		}

		// +------------------------------------------------------------------------------------+
		// |  caretUpdate - CaretListener methods                                               |
		// +------------------------------------------------------------------------------------+
		@Override
		public void caretUpdate( CaretEvent ce )
		{
			if ( !getText().isEmpty() )
			{
				java.awt.EventQueue.invokeLater( new java.lang.Runnable()
				{
					@Override
					public void run()
					{
						synchronized( this )
						{
							int dSpinnerMax = ( ( OwlNumberSpinner )m_jSpinner ).getMaximum();

							Integer jValue = Integer.decode( getText() );

							if ( jValue.intValue() > dSpinnerMax )
							{
								m_jSpinner.setValue( dSpinnerMax );
							}
							else
							{
								m_jSpinner.setValue( jValue );
							}
						}
					}
				} );
			}
		}

		// +------------------------------------------------------------------------------------+
		// |  focusGained - FocusListener method                                                |
		// +------------------------------------------------------------------------------------+
		@Override
		public void focusGained( FocusEvent fe )
		{
			selectAll();
		}
	
		// +------------------------------------------------------------------------------------+
		// |  focusLost - FocusListener method                                                  |
		// +------------------------------------------------------------------------------------+
		@Override
		public void focusLost( FocusEvent fe )
		{
			select( 0, 0 );
		}
	}
}
