package owl.main.remote;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.util.ArrayList;

import javax.swing.JComboBox;
import javax.swing.JOptionPane;

import org.apache.commons.digester3.Digester;

import owl.main.MainApp;


public class ServerDigester
{
	// +--------------------------------------------------------------------+
	// |  writeXml                                                          |
	// +--------------------------------------------------------------------+
	// |  Write all the server data to an xml file. File is overwritten and |
	// |  has the following format:                                         |
	// |                                                                    |
	// | /ServerList                                                        |
	// | /ServerList/server                                                 |
	// | /ServerList/server/ip                                              |
	// | /ServerList/server/description                                     |
	// +--------------------------------------------------------------------+
	public void writeXml( String xmlFile, JComboBox serverComboBox )
	{
		FileWriter fw = null;

		try
		{
			fw = new FileWriter( new File( xmlFile ) );
			fw.append( "<?xml version=\"1.0\"?>\n\n" );
			fw.append( "<ServerList>\n" );

			for ( int i=0; i<serverComboBox.getItemCount(); i++ )
			{
				String[] tokens = ( ( String )serverComboBox.getItemAt( i ) ).split( "-" );
	
				fw.append( "\n\t<server>" );
				fw.append( "\n\t\t<ip>" + tokens[ 0 ].trim() + "</ip>" );
				fw.append( "\n\t\t<description>" + tokens[ 1 ].trim() + "</description>" );
				fw.append( "\n\t</server>\n" );
			}
	
			fw.append( "\n</ServerList>\n\n" );
			fw.close();
		}
		catch ( Exception e )
		{
			MainApp.error( e.getMessage() );
		}
		finally
		{
			try
			{
				if ( fw != null ) { fw.close(); }
			}
			catch ( Exception e ) {}
		}
	}

	// +--------------------------------------------------------------------+
	// |  readXml                                                           |
	// +--------------------------------------------------------------------+
	// |  Reads the and parses the server xml file with the following       |
	// |  format:                                                           |
	// |                                                                    |
	// | /ServerList                                                        |
	// | /ServerList/server                                                 |
	// | /ServerList/server/ip                                              |
	// | /ServerList/server/description                                     |
	// +--------------------------------------------------------------------+
	public ArrayList<ServerElement> readXml( String xmlFile )
	{
		ArrayList<ServerElement> list = null;

		try
		{
			Digester digester = new Digester();

			digester.setValidating( false );

			digester.addObjectCreate( "ServerList", "java.util.ArrayList" );
			digester.addObjectCreate( "ServerList/server", "owl.main.remote.ServerElement" );
			digester.addSetNext( "ServerList/server", "add", "owl.main.remote.ServerElement" );
			digester.addCallMethod( "ServerList/server/ip", "setIp", 0 );
			digester.addCallMethod( "ServerList/server/description", "setDescription", 0 );

			list = digester.parse( new FileInputStream( new File( xmlFile ) ) );
		}
		catch ( Exception e )
		{
			JOptionPane.showMessageDialog( null,
										   "Failed to properly parse SERVER XML file! " +
										   e.getMessage(),
										   "Server XML Parser",
										   JOptionPane.ERROR_MESSAGE);

			list = null;
		}

		return list;
	}
}
