package owl.main.setup;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.lang.reflect.Method;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;

import arc.api.device.ArcDeviceJAPI;

import owl.CCParams.CCScriptEvent;
import owl.CCParams.CCScriptListener;
import owl.gui.scriptable.ScriptableTextButton;
import owl.gui.utils.FunctorRunnable;
import owl.gui.utils.OwlButtonFactory;
import owl.main.MainApp;
import owl.main.device.DeviceConnect;
import owl.main.device.DeviceEvent;
import owl.main.device.DeviceListener;
import owl.main.owltypes.OwlPanel;


public class CameraPanel extends OwlPanel
implements ActionListener, DeviceListener, SetupListener, CCScriptListener
{
	private static final long serialVersionUID = 1L;

	//--------------------------------------------------------------------------
    //   Constants:
    //--------------------------------------------------------------------------
	private final ImageIcon OPEN_SHUTTER_ICON	=	new ImageIcon( MainApp.getBitmapPath() + "OpenShutter.gif" );
	private final ImageIcon CLOSE_SHUTTER_ICON	=	new ImageIcon( MainApp.getBitmapPath() + "CloseShutter.gif" );
	private final ImageIcon CLEAR_ARRAY_ICON	=	new ImageIcon( MainApp.getBitmapPath() + "ClearArray.gif" );
	private final ImageIcon IDLE_ARRAY_ICON		=	new ImageIcon( MainApp.getBitmapPath() + "Idle.gif" );
	private final ImageIcon STOP_ARRAY_ICON		=	new ImageIcon( MainApp.getBitmapPath() + "StopIdle.gif" );
	private final ImageIcon POWER_ON_ICON		=	new ImageIcon( MainApp.getBitmapPath() + "PowerOn.gif" );
	private final ImageIcon POWER_OFF_ICON		=	new ImageIcon( MainApp.getBitmapPath() + "PowerOff.gif" );
	private final ImageIcon RESET_CTLR_ICON		=	new ImageIcon( MainApp.getBitmapPath() + "ResetCtlr.gif" );
	private final ImageIcon RESET_PCI_ICON		=	new ImageIcon( MainApp.getBitmapPath() + "ResetPCI.gif" );
	private final ImageIcon TEMPERATURE_ICON	=	new ImageIcon( MainApp.getBitmapPath() + "Temperature2.gif" );
	private final ImageIcon TEMPTURE_PLOT_ICON	=	new ImageIcon( MainApp.getBitmapPath() + "TemperaturePlot.gif" );

	private final String OPEN_SHUTTER_ACTION	=	"Open Camera Shutter";
	private final String SHUT_SHUTTER_ACTION	=	"Close Camera Shutter";
	private final String CLEAR_ARRAY_ACTION		=	"Clear Camera Array";
	private final String IDLE_ARRAY_ACTION		=	"Idle Camera Array";
	private final String STOP_ARRAY_ACTION		=	"Stop Camera Idle";
	private final String POWER_ON_ACTION		=	"Power Controller On";
	private final String POWER_OFF_ACTION		=	"Power Controller Off";
	private final String RESET_CTLR_ACTION		=	"Reset Controller";
	private final String RESET_PCI_ACTION		=	"Reset PCI(e)";
	private final String TEMPERATURE_ACTION		=	"Read Camera Temperature";
	private final String TEMPTURE_PLOT_ACTION	=	"Show Temperature Plot";
	private final String SETUP_STD_ACTION		=	"Perform Standard Setup Action";

	private final String SETUP_SCRIPT_PREF		=	"Script";
	private final String SETUP_USE_SCRIPT_PREF	=	"UseScript";

	//--------------------------------------------------------------------------
    //   Private Variables:
    //--------------------------------------------------------------------------
	private SetupFrame				m_setupFrame;
	private ScriptableTextButton	m_setupButton;
	private TemperaturePlotFrame	m_tempturePlotFrame;
	private JButton     			m_shutterButton;
	private JButton     			m_idleArrayButton;
	private JButton     			m_powerButton;
    private Thread					m_quickActionThread;
    private JLabel					m_rowsValueLabel;
    private JLabel					m_colsValueLabel;
    private JLabel					m_tmptureValueLabel;
    private JLabel					m_tmptureDNLabel;


	//--------------------------------------------------------------------------
    //   Constructor:
    //--------------------------------------------------------------------------
	public CameraPanel()
	{
		super( "Controller" );

		m_setupFrame = new SetupFrame();
		SetupFrame.addSetupListener( this );
		DeviceConnect.addDeviceListener( this );

		m_tempturePlotFrame = null;

		try
		{
			m_setupButton = new ScriptableTextButton( "Setup", this, SETUP_STD_ACTION );
			m_setupButton.setName( "CameraPanelButton" );
			m_setupButton.setPreferredSize( new Dimension( 65, 20 ) );
			m_setupButton.setAlignmentY( java.awt.Component.TOP_ALIGNMENT );

			Class<?>[] paramClasses = { m_setupFrame.getClass() };
			Method callbackMethod = this.getClass().getDeclaredMethod( "setupScriptCallback", paramClasses );
			Object[] callbackObjects = { m_setupFrame };

			m_setupButton.setScriptCallbackMethod( callbackMethod, callbackObjects );
		}
		catch ( Exception e )
		{
			MainApp.error( e.toString() );
		}

		JPanel setupPanel = new JPanel();
		setupPanel.setLayout( new BoxLayout( setupPanel, BoxLayout.X_AXIS ) );
		setupPanel.setAlignmentX( java.awt.Component.LEFT_ALIGNMENT );
		setupPanel.setAlignmentY( java.awt.Component.TOP_ALIGNMENT );
		setupPanel.add( m_setupButton );

		JLabel rowsLabel = new JLabel( "Rows: " );
		m_rowsValueLabel   = new JLabel( "0" );

		JLabel colsLabel = new JLabel( "Cols: " );
		m_colsValueLabel   = new JLabel( "0" );

		JPanel dimPanel = new JPanel();
		dimPanel.setLayout( new FlowLayout( FlowLayout.LEFT, 10, 0 ) );
		dimPanel.setAlignmentY( java.awt.Component.TOP_ALIGNMENT );
		dimPanel.add( rowsLabel );
		dimPanel.add( m_rowsValueLabel );
		dimPanel.add( colsLabel );
		dimPanel.add( m_colsValueLabel );

		JLabel tmptureLabel = new JLabel( "Temp ( C ): " );
		m_tmptureValueLabel = new JLabel( "000.0" );
		m_tmptureDNLabel = new JLabel( "[0000 DN]" );

		JPanel tmpPanel = new JPanel();
		tmpPanel.setLayout( new FlowLayout( FlowLayout.LEFT ) );
		tmpPanel.setAlignmentY( java.awt.Component.TOP_ALIGNMENT );
		tmpPanel.add( Box.createHorizontalStrut( 1 ) );
		tmpPanel.add( tmptureLabel );
		tmpPanel.add( m_tmptureValueLabel );
		tmpPanel.add( m_tmptureDNLabel );

		JPanel dimTmpPanel = new JPanel();
		dimTmpPanel.setLayout( new BoxLayout( dimTmpPanel, BoxLayout.Y_AXIS ) );
		dimTmpPanel.setAlignmentY( java.awt.Component.TOP_ALIGNMENT );
		dimTmpPanel.add( dimPanel );
		dimTmpPanel.add( tmpPanel );

		JPanel actPanel = createActionPanel();
		actPanel.setAlignmentX( java.awt.Component.LEFT_ALIGNMENT );
		setupPanel.add( dimTmpPanel );

		setLayout( new BoxLayout( this, BoxLayout.Y_AXIS ) );
		setAlignmentY( java.awt.Component.TOP_ALIGNMENT );
		setAlignmentX( java.awt.Component.LEFT_ALIGNMENT );
		add( setupPanel );
		add( actPanel );
		add( Box.createVerticalGlue() );
		add( Box.createVerticalGlue() );
		add( Box.createVerticalGlue() );
		add( Box.createVerticalGlue() );
		add( Box.createVerticalGlue() );
		add( Box.createVerticalGlue() );
		add( Box.createVerticalGlue() );
		add( Box.createVerticalGlue() );
		add( Box.createVerticalGlue() );
		add( Box.createVerticalGlue() );
		add( Box.createVerticalGlue() );
		add( Box.createVerticalGlue() );
		add( Box.createVerticalGlue() );
		add( Box.createVerticalGlue() );
		add( Box.createVerticalGlue() );
		add( Box.createVerticalGlue() );
		add( Box.createVerticalGlue() );
		add( Box.createVerticalGlue() );
		add( Box.createVerticalGlue() );
		add( Box.createVerticalGlue() );
		add( Box.createVerticalGlue() );
		add( Box.createVerticalGlue() );
		add( Box.createVerticalGlue() );

		//  Needed to test if the runnable is still alive
		// +----------------------------------------------
		m_quickActionThread = null;

		loadPreferences();
	}

	public static void setupScriptCallback( SetupFrame param )
	{
		if ( ArcDeviceJAPI.IsControllerConnected() )
		{
			SetupFrame.callSetupListeners( param );
		}
		else
		{
			MainApp.error( "No controller connected to system!" );
		}
	}

	// +--------------------------------------------------------------------+
	// |  getSetupFrame                                                     |
	// +--------------------------------------------------------------------+
	// |  Returns a reference to the setup window.                          |
	// +--------------------------------------------------------------------+
	public SetupFrame getSetupFrame()
	{
		return m_setupFrame;
	}

	public int getRowLabelValue()
	{
		int val = 0;

		try {
			val = Integer.parseInt( m_rowsValueLabel.getText() );
		}
		catch ( Exception e ) { MainApp.error( e.getMessage() ); }

		return val;
	}

	public int getColLabelValue()
	{
		int val = 0;

		try {
			val = Integer.parseInt( m_colsValueLabel.getText() );
		}
		catch ( Exception e ) { MainApp.error( e.getMessage() ); }

		return val;
	}

	private JPanel createActionPanel()
	{
		JPanel topPanel = new JPanel();
		topPanel.setLayout( new FlowLayout( FlowLayout.LEFT, 2, 0 ) );
		topPanel.setBorder( new TitledBorder( "Quick Actions" ) );

		Dimension dim	= new Dimension( 35, 35 );
		Dimension dim2	= new Dimension( 20, 35 );

		m_shutterButton   = OwlButtonFactory.create( OPEN_SHUTTER_ICON, OPEN_SHUTTER_ACTION, dim, this );
		m_idleArrayButton = OwlButtonFactory.create( IDLE_ARRAY_ICON, IDLE_ARRAY_ACTION,     dim, this );
		m_powerButton     = OwlButtonFactory.create( POWER_ON_ICON, POWER_ON_ACTION,         dim, this );

		JButton clearArrayButton  = OwlButtonFactory.create( CLEAR_ARRAY_ICON,   CLEAR_ARRAY_ACTION,   dim,  this );
		JButton resetCtlrButton   = OwlButtonFactory.create( RESET_CTLR_ICON,    RESET_CTLR_ACTION,    dim,  this );
		JButton resetPCIButton    = OwlButtonFactory.create( RESET_PCI_ICON,     RESET_PCI_ACTION,     dim,  this );
		JButton tmptureButton     = OwlButtonFactory.create( TEMPERATURE_ICON,   TEMPERATURE_ACTION,   dim2, this );
		JButton tmpturePlotButton = OwlButtonFactory.create( TEMPTURE_PLOT_ICON, TEMPTURE_PLOT_ACTION, dim2, this );

		topPanel.add( tmptureButton );
		topPanel.add( tmpturePlotButton );
		topPanel.add( m_shutterButton );
		topPanel.add( clearArrayButton );
		topPanel.add( m_idleArrayButton );
		topPanel.add( m_powerButton );
		topPanel.add( resetCtlrButton );
		topPanel.add( resetPCIButton );

		return topPanel;
	}

	@Override
	public void actionPerformed( ActionEvent aEvent )
	{
		String action = aEvent.getActionCommand();

		if ( action.equals( SETUP_STD_ACTION ) )
		{
			m_setupFrame.setVisible( true );
		}

		else if ( action.equals( OPEN_SHUTTER_ACTION ) )
		{
			if ( m_quickActionThread == null ||
				 m_quickActionThread.getState() == Thread.State.TERMINATED )
			{
				Integer[] args = { ArcDeviceJAPI.TIM_ID, ArcDeviceJAPI.OSH };
				m_quickActionThread = new Thread( new FunctorRunnable( 
						/* Method class       */ ArcDeviceJAPI.class,
						/* Method             */ "Command",
						/* Callback class obj */ this,
						/* Callback method    */ "shutterCallback",
						/* Method args Obj[]  */ args,
						/* Display msg box    */ "Open shutter",
						/* Optional msg icon  */ OPEN_SHUTTER_ICON,
						/* Expected reply     */ new Integer( ArcDeviceJAPI.DON ) ) );
				m_quickActionThread.start();
			}
		}

		else if ( action.equals( SHUT_SHUTTER_ACTION ) )
		{
			if ( m_quickActionThread == null ||
				 m_quickActionThread.getState() == Thread.State.TERMINATED )
			{
				Integer[] args = { ArcDeviceJAPI.TIM_ID, ArcDeviceJAPI.CSH };
				m_quickActionThread = new Thread( new FunctorRunnable( 
						/* Method class       */ ArcDeviceJAPI.class,
						/* Method             */ "Command",
						/* Callback class obj */ this,
						/* Callback method    */ "shutterCallback",
						/* Method args Obj[]  */ args,
						/* Display msg box    */ "Close shutter",
						/* Optional msg icon  */ CLOSE_SHUTTER_ICON,
						/* Expected reply     */ new Integer( ArcDeviceJAPI.DON ) ) );
				m_quickActionThread.start();
			}
		}

		else if ( action.equals( CLEAR_ARRAY_ACTION ) )
		{
			if ( m_quickActionThread == null ||
				 m_quickActionThread.getState() == Thread.State.TERMINATED )
			{
				Integer[] args = { ArcDeviceJAPI.TIM_ID, ArcDeviceJAPI.CLR };
				m_quickActionThread = new Thread( new FunctorRunnable( 
						/* Method class       */ ArcDeviceJAPI.class,
						/* Method             */ "Command",
						/* Callback class obj */ null,
						/* Callback method    */ null,
						/* Method args Obj[]  */ args,
						/* Display msg box    */ "Clear camera array",
						/* Optional msg icon  */ CLEAR_ARRAY_ICON,
						/* Expected reply     */ new Integer( ArcDeviceJAPI.DON ) ) );
				m_quickActionThread.start();
			}
		}

		else if ( action.equals( IDLE_ARRAY_ACTION ) )
		{
			if ( m_quickActionThread == null ||
				 m_quickActionThread.getState() == Thread.State.TERMINATED )
			{
				Integer[] args = { ArcDeviceJAPI.TIM_ID, ArcDeviceJAPI.IDL };
				m_quickActionThread = new Thread( new FunctorRunnable( 
						/* Method class       */ ArcDeviceJAPI.class,
						/* Method             */ "Command",
						/* Callback class obj */ this,
						/* Callback method    */ "idleArrayCallback",
						/* Method args Obj[]  */ args,
						/* Display msg box    */ "Idle camera array",
						/* Optional msg icon  */ IDLE_ARRAY_ICON,
						/* Expected reply     */ new Integer( ArcDeviceJAPI.DON ) ) );
				m_quickActionThread.start();
			}
		}

		else if ( action.equals( STOP_ARRAY_ACTION ) )
		{
			if ( m_quickActionThread == null ||
				 m_quickActionThread.getState() == Thread.State.TERMINATED )
			{
				Integer[] args = { ArcDeviceJAPI.TIM_ID, ArcDeviceJAPI.STP };
				m_quickActionThread = new Thread( new FunctorRunnable( 
						/* Method class       */ ArcDeviceJAPI.class,
						/* Method             */ "Command",
						/* Callback class obj */ this,
						/* Callback method    */ "idleArrayCallback",
						/* Method args Obj[]  */ args,
						/* Display msg box    */ "Stop idle camera array",
						/* Optional msg icon  */ STOP_ARRAY_ICON,
						/* Expected reply     */ new Integer( ArcDeviceJAPI.DON ) ) );
				m_quickActionThread.start();
			}
		}

		else if ( action.equals( POWER_ON_ACTION ) )
		{
			if ( m_quickActionThread == null ||
				 m_quickActionThread.getState() == Thread.State.TERMINATED )
			{
				Integer[] args = { ArcDeviceJAPI.TIM_ID, ArcDeviceJAPI.PON };
				m_quickActionThread = new Thread( new FunctorRunnable( 
						/* Method class       */ ArcDeviceJAPI.class,
						/* Method             */ "Command",
						/* Callback class obj */ this,
						/* Callback method    */ "powerCallback",
						/* Method args Obj[]  */ args,
						/* Display msg box    */ "Power on controller",
						/* Optional msg icon  */ POWER_ON_ICON,
						/* Expected reply     */ new Integer( ArcDeviceJAPI.DON ) ) );
				m_quickActionThread.start();
			}
		}

		else if ( action.equals( POWER_OFF_ACTION ) )
		{
			if ( m_quickActionThread == null ||
				 m_quickActionThread.getState() == Thread.State.TERMINATED )
			{
				Integer[] args = { ArcDeviceJAPI.TIM_ID, ArcDeviceJAPI.POF };
				m_quickActionThread = new Thread( new FunctorRunnable( 
						/* Method class       */ ArcDeviceJAPI.class,
						/* Method             */ "Command",
						/* Callback class obj */ this,
						/* Callback method    */ "powerCallback",
						/* Method args Obj[]  */ args,
						/* Display msg box    */ "Power off controller",
						/* Optional msg icon  */ POWER_OFF_ICON,
						/* Expected reply     */ new Integer( ArcDeviceJAPI.DON ) ) );
				m_quickActionThread.start();
			}
		}

		else if ( action.equals( RESET_CTLR_ACTION ) )
		{
			if ( m_quickActionThread == null ||
				 m_quickActionThread.getState() == Thread.State.TERMINATED )
			{
				m_quickActionThread = new Thread( new FunctorRunnable( 
						/* Method class       */ ArcDeviceJAPI.class,
						/* Method             */ "ResetController",
						/* Callback class obj */ null,
						/* Callback method    */ null,
						/* Display msg box    */ "Resetting controller",
						/* Optional msg icon  */ RESET_CTLR_ICON ) );
				m_quickActionThread.start();
			}
		}

		else if ( action.equals( RESET_PCI_ACTION ) )
		{
			m_quickActionThread = new Thread( new FunctorRunnable( 
					/* Method class       */ ArcDeviceJAPI.class,
					/* Method             */ "Reset",
					/* Callback class obj */ null,
					/* Callback method    */ null,
					/* Display msg box    */ "Resetting PCI(e)",
					/* Optional msg icon  */ RESET_PCI_ICON ) );
			m_quickActionThread.start();
		}

		else if ( action.equals( TEMPERATURE_ACTION ) )
		{
			boolean bTempReadout = false;

			try
			{
				bTempReadout =
					ArcDeviceJAPI.IsCCParamSupported( ArcDeviceJAPI.TEMP_SIDIODE );

				if ( !bTempReadout )
				{
					bTempReadout =
						ArcDeviceJAPI.IsCCParamSupported( ArcDeviceJAPI.TEMP_LINEAR );
				}

				if ( bTempReadout )
				{
					if ( m_quickActionThread == null ||
							 m_quickActionThread.getState() == Thread.State.TERMINATED )
					{
						m_quickActionThread = new Thread( new FunctorRunnable( 
										/* Method class       */ ArcDeviceJAPI.class,
										/* Method             */ "GetArrayTemperature",
										/* Callback class obj */ this,
										/* Callback method    */ "temperatureCallback",
										/* Display msg box    */ "Reading Camera Temperature",
										/* Optional msg icon  */ TEMPERATURE_ICON ) );
						m_quickActionThread.start();
					}
				}
				else
				{
					throw new Exception(
							"No temperature readout feature available on controller!" +
							" CCParam: 0x" + Integer.toHexString( ArcDeviceJAPI.GetCCParams() ) );
				}
			}
			catch ( Throwable e )
			{
				MainApp.error( e.getMessage() );
			}
		}

		else if ( action.equals( TEMPTURE_PLOT_ACTION ) )
		{
			if ( m_tempturePlotFrame == null )
			{
				m_tempturePlotFrame =
					new TemperaturePlotFrame( "Array Temperature", true );
			}

			m_tempturePlotFrame.setVisible( true );
		}

		else
		{
			super.actionPerformed( aEvent );
		}
	}

	// +----------------------------------------------------------------+
	// |  deviceChanged                                                 |
	// +----------------------------------------------------------------+
	// |  DeviceListener callback                                       |
	// +----------------------------------------------------------------+
	@Override
	public void deviceChanged( DeviceEvent event )
	{
		m_rowsValueLabel.setText( "0" );
		m_colsValueLabel.setText( "0" );
	}

	// +----------------------------------------------------------------+
	// |  setupChanged                                                  |
	// +----------------------------------------------------------------+
	// |  SetupListener callback                                        |
	// +----------------------------------------------------------------+
	@Override
	public void setupChanged( SetupEvent event )
	{
		try
		{
			// Read and update the current image size display
			//
			// NOTE: join() is used because this thread will
			// contend with the RCC command that is issued by
			// the CCParamPanel and can result in out-of-order
			// or missed replies that cause incorrect row and
			// column values.
			// ------------------------------------------------
			if ( event.bSetupOk )
			{
				m_rowsValueLabel.setText( Integer.toString( event.dRows ) );
				m_colsValueLabel.setText( Integer.toString( event.dCols ) );
			}

			// Adjust power on/off and idle array buttons
			// ------------------------------------------------
			if ( event.bPowerOn )
			{
				m_powerButton.setActionCommand( POWER_OFF_ACTION );
				m_powerButton.setToolTipText( POWER_OFF_ACTION );
				m_powerButton.setIcon( POWER_OFF_ICON );

				m_idleArrayButton.setIcon( STOP_ARRAY_ICON );
				m_idleArrayButton.setToolTipText( STOP_ARRAY_ACTION );
				m_idleArrayButton.setActionCommand( STOP_ARRAY_ACTION );
			}
			else
			{
				m_powerButton.setActionCommand( POWER_ON_ACTION );
				m_powerButton.setToolTipText( POWER_ON_ACTION );
				m_powerButton.setIcon( POWER_ON_ICON );

				m_idleArrayButton.setIcon( IDLE_ARRAY_ICON );
				m_idleArrayButton.setToolTipText( IDLE_ARRAY_ACTION );
				m_idleArrayButton.setActionCommand( IDLE_ARRAY_ACTION );
			}
		}
		catch ( Exception e )
		{
			MainApp.error( "CameraPanel.setupChanged error: " +
							e.toString() );
		}
	}

	// +----------------------------------------------------------------+
	// |  CCScriptChanged                                               |
	// +----------------------------------------------------------------+
	// |  CCScriptListener callback                                     |
	// +----------------------------------------------------------------+
	@Override
	public void CCScriptChanged( CCScriptEvent event )
	{
		//  Check the current image dimension on the controller
		// +------------------------------------------------------+
		try
		{
			int[] imgSize = ArcDeviceJAPI.GetImageSize();

			m_rowsValueLabel.setText( Integer.toString( imgSize[ 0 ] ) );
			m_colsValueLabel.setText( Integer.toString( imgSize[ 1 ] ) );
		}
		catch ( Throwable e )
		{
			MainApp.error( "Failed to read current image dimensions" +
						   " from the controller. " + e.getMessage() );
		}
	}

	public void shutterCallback( int retVal )
	{
		if ( retVal != ArcDeviceJAPI.DON ) { return; }

		if ( m_shutterButton.getActionCommand().equals( OPEN_SHUTTER_ACTION ) )
		{
			m_shutterButton.setIcon( CLOSE_SHUTTER_ICON );
			m_shutterButton.setToolTipText( SHUT_SHUTTER_ACTION );
			m_shutterButton.setActionCommand( SHUT_SHUTTER_ACTION );
		}
		else
		{
			m_shutterButton.setIcon( OPEN_SHUTTER_ICON );
			m_shutterButton.setToolTipText( OPEN_SHUTTER_ACTION );
			m_shutterButton.setActionCommand( OPEN_SHUTTER_ACTION );
		}
	}

	public void idleArrayCallback( int retVal )
	{
		if ( retVal != ArcDeviceJAPI.DON ) return;

		if ( m_idleArrayButton.getActionCommand().equals( IDLE_ARRAY_ACTION ) )
		{
			m_idleArrayButton.setIcon( STOP_ARRAY_ICON );
			m_idleArrayButton.setToolTipText( STOP_ARRAY_ACTION );
			m_idleArrayButton.setActionCommand( STOP_ARRAY_ACTION );
		}
		else
		{
			m_idleArrayButton.setIcon( IDLE_ARRAY_ICON );
			m_idleArrayButton.setToolTipText( IDLE_ARRAY_ACTION );
			m_idleArrayButton.setActionCommand( IDLE_ARRAY_ACTION );
		}
	}

	public void powerCallback( int retVal )
	{
		if ( retVal != ArcDeviceJAPI.DON ) return;

		if ( m_powerButton.getActionCommand().equals( POWER_ON_ACTION ) )
		{
			m_powerButton.setIcon( POWER_OFF_ICON );
			m_powerButton.setToolTipText( POWER_OFF_ACTION );
			m_powerButton.setActionCommand( POWER_OFF_ACTION );
		}
		else
		{
			m_powerButton.setIcon( POWER_ON_ICON );
			m_powerButton.setToolTipText( POWER_ON_ACTION );
			m_powerButton.setActionCommand( POWER_ON_ACTION );
		}
	}

	public void temperatureCallback( double val )
	{
		java.text.DecimalFormat df = new java.text.DecimalFormat( "#0.0" );
		m_tmptureValueLabel.setText( df.format( val ) + " C" );

		if ( m_quickActionThread == null ||
			 m_quickActionThread.getState() == Thread.State.RUNNABLE )
		{
				m_quickActionThread = new Thread( new FunctorRunnable( 
							/* Method class       */ ArcDeviceJAPI.class,
							/* Method             */ "GetArrayTemperatureDN",
							/* Callback class obj */ this,
							/* Callback method    */ "temperatureDNCallback",
							/* Display msg box    */ "Reading Temperature DN",
							/* Optional msg icon  */ TEMPERATURE_ICON ) );
				m_quickActionThread.start();
		}

		MainApp.mainFrame.pack();
	}

	public void temperatureDNCallback( double gDN )
	{
		java.text.DecimalFormat df = new java.text.DecimalFormat( "0000" );
		df.setMaximumIntegerDigits( 4 );
		m_tmptureDNLabel.setText( "[" + df.format( gDN ) + " DN]" );
	}

	// +--------------------------------------------------------------------+
	// |  savePreferences                                                   |
	// +--------------------------------------------------------------------+
	// |  Saves the setup button script and state upon exit                 |
	// +--------------------------------------------------------------------+
	@Override
	protected void savePreferences()
	{
		super.savePreferences();

		String  prefKey  = null;
		String  tempStr  = null;
		boolean tempBool = false;

		prefKey = m_setupButton.getName() + SETUP_SCRIPT_PREF;
		tempStr = m_setupButton.getScriptFilename();
		if ( tempStr != null ) { MainApp.getPreferences().put( prefKey, tempStr ); }
	
		prefKey = m_setupButton.getName() + SETUP_USE_SCRIPT_PREF;
		tempBool = m_setupButton.isScriptSelected();
		MainApp.getPreferences().putBoolean( prefKey, tempBool );
	}

	// +--------------------------------------------------------------------+
	// |  loadPreferences                                                   |
	// +--------------------------------------------------------------------+
	// |  Loads the buttons and their scripts upon startup.                 |
	// +--------------------------------------------------------------------+
	protected void loadPreferences()
	{
		if ( m_setupButton != null )
		{
			boolean useScript      = false;
			String  scriptFilename = null;
			String  prefKey        = null;

			prefKey        = m_setupButton.getName() + SETUP_SCRIPT_PREF;
			scriptFilename = MainApp.getPreferences().get( prefKey, null );

			prefKey   = m_setupButton.getName() + SETUP_USE_SCRIPT_PREF;
			useScript = MainApp.getPreferences().getBoolean( prefKey, false );

			if ( scriptFilename != null && !scriptFilename.isEmpty() )
			{
				m_setupButton.setScript( scriptFilename, useScript );
			}
		}
	}
}
