package owl.plot;

import java.awt.Color;

import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;


// +-----------------------------------------------------------------------------+
// | PlotFactory                                                                 |
// +-----------------------------------------------------------------------------+
// | Convenience class for wrapping up a plot into an org.jfree.chart.ChartPanel |
// | that can be added to a window as a component.                               |
// +-----------------------------------------------------------------------------+
public class PlotFactory
{
	// +------------------------------------------------------------------------+
	// |  createXYChart                                                         |
	// +------------------------------------------------------------------------+
	// |  Wraps an XYPlot into an org.jfree.chart.ChartPanel.                   |
	// |                                                                        |
	// |  @param plot          - The XYPlot to wrap.                            |
	// |  @param title         - The plot title ( for the ChartPanel ).         |
	// |  @param background    - The plot and chart background color.           |
	// |  @param bRemoveLegend - 'true' to remove the chart legend.             |
	// +------------------------------------------------------------------------+
	public static ChartPanel
	createXYChart( XYPlot plot, String title, Color background, boolean bRemoveLegend )
	{
		JFreeChart chart = new JFreeChart( plot );
		chart.setBackgroundPaint( background );
		chart.setTitle( title );

		if ( bRemoveLegend )
		{
			chart.removeLegend();
		}

		ChartPanel chartPanel = new ChartPanel( chart );
		chartPanel.setBackground( background );

		return chartPanel;
	}

	// +------------------------------------------------------------------------+
	// |  createHistogramChart                                                  |
	// +------------------------------------------------------------------------+
	// |  Wraps a HistogramPlot into an org.jfree.chart.ChartPanel.             |
	// |                                                                        |
	// |  @param plot          - The HistogramPlot to wrap.                     |
	// |  @param title         - The plot title ( for the ChartPanel ).         |
	// |  @param background    - The plot and chart background color.           |
	// |  @param bRemoveLegend - 'true' to remove the chart legend.             |
	// +------------------------------------------------------------------------+
	public static ChartPanel
	createHistogramChart( HistogramPlot plot, String title, Color background, boolean bRemoveLegend )
	{
		JFreeChart chart = new JFreeChart( plot );
		chart.setBackgroundPaint( background );
		chart.setTitle( title );

		if ( bRemoveLegend )
		{
			chart.removeLegend();
		}

		ChartPanel chartPanel = new ChartPanel( chart );
		chartPanel.setBackground( background );

		return chartPanel;
	}
}
